# -*- coding: utf-8 -*-
from typing import List, Dict

from msgbus import MBEasyContext
from old_helper import BaseRepository
from persistence import Connection


class ProductRepository(BaseRepository):

    def __init__(self, mb_context):
        # type: (MBEasyContext) -> None

        super(ProductRepository, self).__init__(mb_context)

    def get_items_names(self, items):
        # type: (List[int]) -> Dict

        def inner_func(conn):
            # type: (Connection) -> Dict

            query = self._ProductNamesQuery
            product_codes = ','.join(map(str, items))
            query = query.format(product_codes)
            product_names = [(int(x.get_entry(0)), x.get_entry(1)) for x in conn.select(query)]
            return dict(product_names)

        return self.execute_with_connection(inner_func)

    _ProductNamesQuery = """\
        SELECT *
        FROM Product
        WHERE ProductCode IN ({0})
"""



from commons.dto import DefaultHeaderDto
from commons.report import Generator  # noqa
from commons.repository import AccountRepository  # noqa
from reports_app.cashreport.dto import CashReportDto  # noqa
from reports_app.cashreport.dto import CashReportFiltersDto  # noqa
from reports_app.cashreport.dto import CashReportBodyDto
from reports_app.cashreport.dto import CashReportBodyLineDto
from reports_app.cashreport.dto import OrderInfo


class CashReportGenerator(Generator):
    def __init__(self, account_repository, order_repository, cash_report_filter):
        # type: (AccountRepository, OrderRepository, CashReportFiltersDto) -> None
        self.account_repository = account_repository
        self.cash_report_filter = cash_report_filter
        self.order_repository = order_repository

    def generate_data(self):
        header_report_dto = self.generate_header()
        body_report_dto = self.generate_body()
        return CashReportDto(header_report_dto, body_report_dto, self.cash_report_filter.caller_pos_id)

    def generate_header(self):
        if self.cash_report_filter.report_type == 'report_by_session_id':
            date = self.cash_report_filter.initial_date.strftime("%d/%m/%Y")
        else:
            date = self.cash_report_filter.initial_date.strftime("%d/%m/%Y") + " - " + self.cash_report_filter.end_date.strftime("%d/%m/%Y")

        header_report_dto = DefaultHeaderDto(
            self.cash_report_filter.store_id,
            self.cash_report_filter.pos_id,
            self.cash_report_filter.operator_id,
            date,
            self.cash_report_filter.report_type
        )
        return header_report_dto

    def generate_body(self):
        initial_balance = CashReportBodyLineDto('$INITIAL_BALANCE', None, 0.0)
        sales = CashReportBodyLineDto('$SALES', 0, 0.0)
        voided_orders = CashReportBodyLineDto('$VOIDED_ORDERS', 0, 0.0)
        discounts = CashReportBodyLineDto('$DISCOUNTS', None, None)
        donations = CashReportBodyLineDto('$DONATIONS', 0, 0.0)
        liquid_sales = CashReportBodyLineDto('$LIQUID_SALES', 0, 0.0)
        tender_types = CashReportBodyLineDto('$TENDER_TYPES', None, None)
        cash_out = CashReportBodyLineDto('$CASH_OUT', 0, 0.0)
        cash_in = CashReportBodyLineDto('$CASH_IN', 0, 0.0)
        value_in_drawer = CashReportBodyLineDto('$VALUE_IN_DRAWER', None, 0.0)

        self.parse_account_db(cash_in, cash_out, initial_balance)
        self.parse_order_db(cash_in, cash_out, initial_balance, sales, voided_orders, liquid_sales, donations, value_in_drawer)

        return CashReportBodyDto([initial_balance,
                                  sales,
                                  voided_orders,
                                  discounts,
                                  donations,
                                  liquid_sales,
                                  tender_types,
                                  cash_out,
                                  cash_in,
                                  value_in_drawer], 'report_by_business_period')

    def parse_order_db(self, cash_in, cash_out, initial_balance, sales, voided_orders, liquid_sales, donations, value_in_drawer):
        orders_info = self.get_order_db_info()

        for item in orders_info.paid_orders:
            liquid_sales.quantity += 1
            liquid_sales.value += item.total
        if orders_info.voided_orders is not None:
            for item in orders_info.voided_orders:
                voided_orders.quantity += 1
                voided_orders.value += item.total
        for item in orders_info.paid_orders:
            if item.donation > 0:
                donations.quantity += 1
                donations.value += item.donation
        if orders_info.voided_orders is not None:
            sales.quantity = liquid_sales.quantity + voided_orders.quantity
            sales.value = liquid_sales.value + voided_orders.value

        value_in_drawer.value = initial_balance.value + cash_in.value - cash_out.value
        for item in orders_info.paid_orders:
            for item_tender in item.tenders:
                if item_tender.type == 0:
                    value_in_drawer.value += item_tender.value
            value_in_drawer.value -= item.change_amount

    def get_value_in_drawer(self):
        initial_balance = CashReportBodyLineDto(None, None, 0.0)
        cash_out = CashReportBodyLineDto(None, 0, 0.0)
        cash_in = CashReportBodyLineDto(None, 0, 0.0)

        self.cash_report_filter.report_type = 'report_by_session_id'

        self.parse_account_db(cash_in, cash_out, initial_balance)
        orders_info = self.get_order_db_info()

        value_in_drawer = initial_balance.value + cash_in.value - cash_out.value
        for item in orders_info.paid_orders:
            for item_tender in item.tenders:
                if item_tender.type == 0:
                    value_in_drawer += item_tender.value
            value_in_drawer -= item.change_amount

        return value_in_drawer

    def get_order_db_info(self):
        paid_orders = None
        voided_orders = None
        if self.cash_report_filter.report_type == 'report_by_real_date':
            paid_orders = self.order_repository.get_paid_orders_by_real_date(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id)
            voided_orders = self.order_repository.get_voided_orders_by_real_date(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id)
        if self.cash_report_filter.report_type == 'report_by_business_period':
            paid_orders = self.order_repository.get_paid_orders_by_business_period(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id)
            voided_orders = self.order_repository.get_voided_orders_by_business_period(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id)
        if self.cash_report_filter.report_type == 'report_by_session_id':
            paid_orders = self.order_repository.get_paid_orders_by_session_id(
                self.cash_report_filter.session_id)
            voided_orders = self.order_repository.get_voided_orders_by_session_id(
                self.cash_report_filter.session_id)
        if self.cash_report_filter.report_type == 'report_by_xml':
            paid_orders = self.order_repository.get_paid_orders_by_xml(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date)
            voided_orders = None

        return OrderInfo(paid_orders, voided_orders, "todo")

    def parse_account_db(self, cash_in, cash_out, initial_balance):
        account_info = self.get_account_db_info()

        if account_info is not None:
            for item in account_info:
                if item.transfer_cash_in_cash_out == 1 and initial_balance.value == 0.0:
                    initial_balance.value = item.transfer_value
                elif item.transfer_cash_in_cash_out == 3 or item.transfer_cash_in_cash_out == 1:
                    cash_in.quantity += 1
                    cash_in.value += item.transfer_value
                elif item.transfer_cash_in_cash_out == 4:
                    cash_out.quantity += 1
                    cash_out.value += item.transfer_value

    def get_account_db_info(self):
        account_info = None
        if self.cash_report_filter.report_type == 'report_by_real_date':
            account_info = self.account_repository.get_transfers_by_real_date(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id,
                self.cash_report_filter.report_type)
        if self.cash_report_filter.report_type == 'report_by_business_period':
            account_info = self.account_repository.get_transfers_by_business_period(
                self.cash_report_filter.initial_date,
                self.cash_report_filter.end_date,
                self.cash_report_filter.operator_id,
                self.cash_report_filter.pos_id,
                self.cash_report_filter.report_type)
        if self.cash_report_filter.report_type == 'report_by_session_id':
            account_info = self.account_repository.get_transfers_by_session_id(
                self.cash_report_filter.session_id,
                self.cash_report_filter.report_type)
        if self.cash_report_filter.report_type == 'report_by_xml':
            account_info = None
        return account_info



from abc import ABCMeta, abstractmethod

from messagebus import AckMessage, NakMessage
from messageprocessorutil import UuidMessageProcessor
from production.box import ViewBox
from production.command import Command, CommandType
from production.model.request import ChangeProdStateRequest
from typing import List, Dict


class CommandProcessor(UuidMessageProcessor):
    __metaclass__ = ABCMeta

    def __init__(self, view_boxes):
        # type: (List[ViewBox]) -> None
        super(CommandProcessor, self).__init__()
        self.view_boxes = {}  # type: Dict[str, ViewBox]
        for box in view_boxes:
            self.view_boxes[box.get_view_name()] = box

    def call_business(self, request):
        # type: (ChangeProdStateRequest) -> None
        if request.view not in self.view_boxes:
            return None

        if hasattr(request, "order_id") and request.order_id is not None:
            request.order_id = str(request.order_id)

        view_box = self.view_boxes[request.view]
        view_box.handle_view_command(Command(self.get_command_type(), request))

    def format_response(self, message_bus, message, event, input_obj, result):
        message_bus.reply_message(message, AckMessage())

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, NakMessage())

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, NakMessage())

    @abstractmethod
    def get_command_type(self):
        # type: () -> CommandType
        raise NotImplementedError()



from old_helper import BaseRepository
from msgbus import MBEasyContext
from persistence import Connection
from typing import Dict, Tuple


class FiscalParameterRepository(BaseRepository):
    def __init__(self, mbcontext):
        # type: (MBEasyContext) -> None
        super(FiscalParameterRepository, self).__init__(mbcontext)

    def get_fiscal_parameters(self, loader_params):
        def inner_func(conn):
            # type: (Connection) -> Dict[Tuple[int, unicode], unicode]
            query = "select ProductCode, ParamName, ParamValue from FiscalParameter"
            return {(str(x.get_entry(0)), x.get_entry(1)): x.get_entry(2) for x in conn.select(query)}

        fiscal_params = self.execute_with_connection(inner_func, service_name="FiscalPersistence")

        if loader_params:
            for product in loader_params:
                product_code = product['PRODUCT_CODE']
                for param in product:
                    if param == 'PRODUCT_CODE':
                        continue
                    result = {(str(product_code), str(param)): str(product[param])}
                    fiscal_params.update(result)

        return fiscal_params



from customer_services.model import Customer, InvalidRequestException, CustomerNonExistsException
from customer_services.model.repository import CustomerRepository


class GetCustomerService(object):
    def __init__(self, customer_repository):
        # type: (CustomerRepository) -> None
        
        self.customer_repository = customer_repository
    
    def get_customer(self, phone):
        # type: (str) -> Customer

        if not phone:
            raise InvalidRequestException()

        customer = self.customer_repository.get_customer(phone)
        if not customer:
            raise CustomerNonExistsException()

        return customer



# defusedxml
#
# Copyright (c) 2013 by Christian Heimes <christian@python.org>
# Licensed to PSF under a Contributor Agreement.
# See https://www.python.org/psf/license for licensing details.
"""Defused xml.dom.expatbuilder
"""
from __future__ import print_function, absolute_import

from xml.dom.expatbuilder import ExpatBuilder as _ExpatBuilder
from xml.dom.expatbuilder import Namespaces as _Namespaces

from .common import DTDForbidden, EntitiesForbidden, ExternalReferenceForbidden

__origin__ = "xml.dom.expatbuilder"


class DefusedExpatBuilder(_ExpatBuilder):
    """Defused document builder"""

    def __init__(
        self, options=None, forbid_dtd=False, forbid_entities=True, forbid_external=True
    ):
        _ExpatBuilder.__init__(self, options)
        self.forbid_dtd = forbid_dtd
        self.forbid_entities = forbid_entities
        self.forbid_external = forbid_external

    def defused_start_doctype_decl(self, name, sysid, pubid, has_internal_subset):
        raise DTDForbidden(name, sysid, pubid)

    def defused_entity_decl(
        self, name, is_parameter_entity, value, base, sysid, pubid, notation_name
    ):
        raise EntitiesForbidden(name, value, base, sysid, pubid, notation_name)

    def defused_unparsed_entity_decl(self, name, base, sysid, pubid, notation_name):
        # expat 1.2
        raise EntitiesForbidden(name, None, base, sysid, pubid, notation_name)  # pragma: no cover

    def defused_external_entity_ref_handler(self, context, base, sysid, pubid):
        raise ExternalReferenceForbidden(context, base, sysid, pubid)

    def install(self, parser):
        _ExpatBuilder.install(self, parser)

        if self.forbid_dtd:
            parser.StartDoctypeDeclHandler = self.defused_start_doctype_decl
        if self.forbid_entities:
            # if self._options.entities:
            parser.EntityDeclHandler = self.defused_entity_decl
            parser.UnparsedEntityDeclHandler = self.defused_unparsed_entity_decl
        if self.forbid_external:
            parser.ExternalEntityRefHandler = self.defused_external_entity_ref_handler


class DefusedExpatBuilderNS(_Namespaces, DefusedExpatBuilder):
    """Defused document builder that supports namespaces."""

    def install(self, parser):
        DefusedExpatBuilder.install(self, parser)
        if self._options.namespace_declarations:
            parser.StartNamespaceDeclHandler = self.start_namespace_decl_handler

    def reset(self):
        DefusedExpatBuilder.reset(self)
        self._initNamespaces()


def parse(file, namespaces=True, forbid_dtd=False, forbid_entities=True, forbid_external=True):
    """Parse a document, returning the resulting Document node.

    'file' may be either a file name or an open file object.
    """
    if namespaces:
        build_builder = DefusedExpatBuilderNS
    else:
        build_builder = DefusedExpatBuilder
    builder = build_builder(
        forbid_dtd=forbid_dtd, forbid_entities=forbid_entities, forbid_external=forbid_external
    )

    if isinstance(file, str):
        fp = open(file, "rb")
        try:
            result = builder.parseFile(fp)
        finally:
            fp.close()
    else:
        result = builder.parseFile(file)
    return result


def parseString(
    string, namespaces=True, forbid_dtd=False, forbid_entities=True, forbid_external=True
):
    """Parse a document from a string, returning the resulting
    Document node.
    """
    if namespaces:
        build_builder = DefusedExpatBuilderNS
    else:
        build_builder = DefusedExpatBuilder
    builder = build_builder(
        forbid_dtd=forbid_dtd, forbid_entities=forbid_entities, forbid_external=forbid_external
    )
    return builder.parseString(string)



from ._BurnBenefitInteractor import BurnBenefitInteractor
from ._GetBenefitInfoInteractor import GetBenefitInfoInteractor
from ._ApplyBenefitInteractor import ApplyBenefitInteractor
from ._UnlockBenefitInteractor import UnlockBenefitInteractor
from ._OrderModifiedReopenedInteractor import OrderModifiedReopenedInteractor
from ._OrderModifiedPaidInteractor import OrderModifiedPaidInteractor
from ._OrderModifiedVoidInteractor import OrderModifiedVoidInteractor
from ._OrderModifiedVoidLineInteractor import OrderModifiedVoidLineInteractor
from ._BenefitRetryInteractor import BenefitRetryInteractor



import sysactions
from actions.functions.do_recall_order import do_recall_order
from systools import sys_log_exception
from utils import get_void_reason_id, void_order


@sysactions.action
def do_void_stored_order(pos_id, order_id="", orig_id="", void_reason=6, *args):
    try:
        void_reason = get_void_reason_id(pos_id, void_reason)
        if not void_reason:
            return

        do_recall_order(pos_id, order_id, check_date=False)
        void_order(pos_id, void_reason=void_reason)

        sysactions.show_messagebox(pos_id, "$STORED_ORDER_IS_VOIDED")
    except Exception as _:
        sysactions.show_messagebox(pos_id, "$ERROR_VOIDING_STORED_ORDER", "$ERROR")
        sys_log_exception("Erro ao apagar pedido salvo")



# -*- coding: utf-8 -*-

import logging
import unicodedata

from typing import Dict, Any

from old_helper import validar_cnpj
from helper import remove_accents
from pos_model import Order, TenderType

from models import PosTenderToNfeTender, NfeTender, PosTenderBrandToNfeCardBrand
from _NfceXmlPartBuilder import NfceXmlPartBuilder

logger = logging.getLogger("FiscalWrapper")


class PagBuilder(NfceXmlPartBuilder):

    def __init__(self, versao_ws):
        # type: (int) -> None
        self.versao_ws = versao_ws

    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode

        if self.versao_ws not in (1, 3):
            xml = "<pag>"
        else:
            xml = ""
        for tender in order.tenders:
            tender_type = PosTenderToNfeTender.convert(tender.type)

            cnpj = validar_cnpj(tender.cnpj_auth)

            if tender.type in [TenderType.credit, TenderType.debit] and cnpj is False:
                tender_type = NfeTender.outros

            if self.versao_ws not in (1, 3):
                xml += "<detPag>"
            else:
                xml += "<pag>"

            if self.versao_ws in (1, 3):
                total_amount = tender.amount - tender.change
            else:
                total_amount = tender.amount

            xml += "<tPag>{:>02}</tPag>".format(tender_type)

            if tender_type == NfeTender.outros:
                tender_desc = tender.descr or tender.type
                xml += "<xPag>{}</xPag>".format(remove_accents(tender_desc))

            xml += "<vPag>{:>0.2f}</vPag>".format(total_amount)

            if tender_type in [NfeTender.cartao_credito, NfeTender.cartao_debito] and cnpj is not False:
                card_brand = PosTenderBrandToNfeCardBrand.convert(tender.bandeira)
                xml += "<card><tpIntegra>1</tpIntegra><CNPJ>{0}</CNPJ><tBand>{1}</tBand><cAut>{2}</cAut></card>" \
                    .format(tender.cnpj_auth.strip(), card_brand, tender.auth_code.strip())

            if self.versao_ws not in (1, 3):
                xml += "</detPag>"
            else:
                xml += "</pag>"

            if self.versao_ws not in (1, 3) and tender.change > 0.0:
                xml += "<vTroco>{:>0.2f}</vTroco>".format(tender.change)

        if self.versao_ws not in (1, 3):
            xml += "</pag>"

        return xml



# -*- coding: utf-8 -*-

import time
from xml.etree import cElementTree as eTree

from _ModuleLogger import logger
from old_helper import OrderTaker
from nfcedisabler import NfceDisabler, OrderRepository
from pos_model import OrderKey
from posot import OrderTakerException
from requests.exceptions import RequestException
from fiscalpersistence import FiscalDataRepository


class OrderDisabler(object):
    def __init__(self, order_repository, nfce_disabler, order_taker):
        # type: (OrderRepository, NfceDisabler, OrderTaker) -> None
        self.order_repository = order_repository
        self.nfce_disabler = nfce_disabler
        self.order_taker = order_taker

    def disable_all_orders(self, mb_context):
        logger.info("Inutilizando notas")
        order_keys = self.order_repository.get_all_orders_to_disable()
        logger.info("Orders para inutilizar: {0}".format(len(order_keys)))

        for order_key in order_keys:  # type: OrderKey
            pos_id = order_key.pos_id
            try:
                logger.info("Inutilizando order: {0}".format(order_key.order_id))
                order_xml = self.order_taker.get_order_picture(pos_id, order_key.order_id)
                logger.info("Iniciando inutilização da order {0} do pos {1}".format(order_key.order_id, pos_id))
                xml = self.nfce_disabler.disable_fiscal_number(eTree.XML(order_xml))
                self.order_repository.mark_order_disabled(pos_id, order_key.order_id)
                with FiscalDataRepository(mb_context) as fiscal_repository:
                    fiscal_repository.notify_pump(pos_id, order_key.order_id, xml_format='OrderDisabler')
                    fiscal_repository.set_fiscal_xml_custom_property(pos_id, xml, key='DISABLEMENT_FISCAL_XML')

                logger.info("Order inutilizada com sucesso: {0}".format(order_key.order_id))
            except RequestException as _:
                logger.exception("Erro de Conexao com a Sefaz tentando inutulizar nota, tentaremos novamente...")
            except OrderTakerException:
                logger.exception("Order Taker Error, tentaremos novamente...")
            except BaseException as _:
                logger.exception("Erro inutilizando nota, verifique o Layout e os parametros enviados pela Sefaz, não tentaremos novamente.")
                self.order_repository.mark_order_not_disabled(pos_id, order_key.order_id)
            finally:
                time.sleep(5)



from production.box._OrderChangerProductionBox import OrderChangerProductionBox

from ._AllowedForbiddenFilter import AllowedForbiddenFilter


class TagFilterBox(OrderChangerProductionBox):
    def __init__(self, name, sons, allowed_tags, forbidden_tags, allow_combos, logger=None):
        super(TagFilterBox, self).__init__(name, sons, logger)
        self.filter = AllowedForbiddenFilter(allowed_tags, forbidden_tags, allow_combos)

    def change_order(self, order):
        order = self.filter.filter_items(order, lambda x: x.get_tags())
        return order



import json
from ast import literal_eval
from xml.etree import cElementTree as eTree

import sysactions
from _AposA8PrintLine import AposA8PrintLine
from _AposA8Align import AposA8Align
from _AposA8CommandType import AposA8CommandType
from _AposA8Constants import AposA8Constants
from bustoken import create_token
from comp_exceptions import VoidOrderException
from old_helper import F, get_sale_line_priced_items
from msgbus import TK_SYS_NAK
from helper import remove_accents

APOS_DASH = AposA8PrintLine(AposA8Align.CENTER, AposA8Constants.SEPARATOR_DASH, AposA8CommandType.PRINT_TEXT)
APOS_NEWLINE = AposA8PrintLine(AposA8Align.CENTER, "", AposA8CommandType.PRINT_TEXT)
APOS_HASH = AposA8PrintLine(AposA8Align.CENTER, AposA8Constants.SEPARATOR_HASH, AposA8CommandType.PRINT_TEXT)


def add_loyalty_custom_text(report, order, custom_text):
    # type: (list, eTree.Element, dict) -> None
    loyalty_user_name = order.find("CustomOrderProperties/OrderProperty[@key='CUSTOMER_NAME']")
    loyalty_user_cpf = order.find("CustomOrderProperties/OrderProperty[@key='LOYALTY_SYSTEM_USER_CPF']")

    if loyalty_user_name is not None and loyalty_user_cpf is not None:
        name_and_cpf = custom_text["name_cpf"].format(nome_cliente=loyalty_user_name.get("value"),
                                                      cpf_cliente=loyalty_user_cpf.get("value"))
        report.append(AposA8PrintLine(AposA8Align.CENTER, name_and_cpf, AposA8CommandType.PRINT_TEXT))
    elif loyalty_user_cpf is not None:
        cpf = custom_text["cpf"].format(cpf_cliente=loyalty_user_cpf.get("value"))
        report.append(AposA8PrintLine(AposA8Align.CENTER, cpf, AposA8CommandType.PRINT_TEXT))
    else:
        non_participating = custom_text["non_participating"]
        report.append(AposA8PrintLine(AposA8Align.CENTER, non_participating, AposA8CommandType.PRINT_TEXT))

    report.append(APOS_NEWLINE)
    report.append(APOS_DASH)
    report.append(APOS_NEWLINE)


def get_whopper_wifi_data(order):
    whopper_wifi_property = order.find("CustomOrderProperties/OrderProperty[@key='WHOPPER_WIFI_CODE']")
    if whopper_wifi_property is not None:
        whopper_wifi_code = whopper_wifi_property.get("value")[0:4] + " " + whopper_wifi_property.get("value")[4:8]
    else:
        whopper_wifi_code = None
    return whopper_wifi_code


def append_items_header(report):
    product_header = "COD     DESCR          QTD UN    VL UNIT   TOTAL"
    report.append(AposA8PrintLine(AposA8Align.CENTER, product_header, AposA8CommandType.PRINT_TEXT))


def append_and_count_sale_items(report, order, model, tip_config):
    items_amount = 0
    sale_lines = get_sale_line_priced_items(order.findall("SaleLine"))
    un = "un"
    for line in sale_lines:
        item_price, unit_price, item_qty, name, part_code = map(line.get, (
            "correctItemPrice",
            "correctUnitPrice",
            "correctQty",
            "productName",
            "partCode"
        ))
        items_amount += 1
        line = "%-7.7s %-14.14s%5.3f%2.2s%9.02f" % (part_code, name, F(item_qty), un, F(unit_price))
        string = ("%-38.38s %9.02f" % (line, F(item_price)))

        report.append(AposA8PrintLine(AposA8Align.CENTER, string, AposA8CommandType.PRINT_TEXT))

    if order.get("tip", 0) and float(order.get("tip")) > 0:
        order_tip = F(order.get("tip"))
        tip_line = "%-7.7s %-14.14s%5.3f%2.2s%9.02f" % (tip_config['TipCode'],
                                                        sysactions.translate_message(model, "TIP_GIVEN").upper(),
                                                        F(1),
                                                        "un",
                                                        order_tip)
        tip_text = ("%-38.38s %9.02f" % (tip_line, order_tip))
        report.append(AposA8PrintLine(AposA8Align.CENTER, tip_text, AposA8CommandType.PRINT_TEXT))

    return items_amount


def append_whopper_wifi_code(report, model, whopper_wifi_code):
    if whopper_wifi_code is None or model is None:
        return

    whopper_wifi_code_text = sysactions.translate_message(model, "WIFI_PRINTER_MESSAGE")
    report.append(AposA8PrintLine(AposA8Align.CENTER, whopper_wifi_code_text, AposA8CommandType.PRINT_TEXT))

    wifi_access_code = sysactions.translate_message(model, "WIFI_ACCESS_CODE", "%s" % whopper_wifi_code)
    report.append(AposA8PrintLine(AposA8Align.CENTER, wifi_access_code, AposA8CommandType.PRINT_TEXT))


def append_tax_data(report, total_taxes):
    approx_value_text = "Valor aproximado dos tributos deste cupom"
    report.append(AposA8PrintLine(AposA8Align.LEFT, approx_value_text, AposA8CommandType.PRINT_TEXT))

    nacional_federal_value = total_taxes['nacionalfederal']['value']
    estadual_value = total_taxes['estadual']['value']

    total_tax_value = float(nacional_federal_value) + float(estadual_value)
    report.append(AposA8PrintLine(AposA8Align.RIGHT, "R$%7.2f" % total_tax_value, AposA8CommandType.PRINT_TEXT))

    nacional_federal_percent = total_taxes['nacionalfederal']['percent']
    estadual_percent = total_taxes['estadual']['percent']

    federal_tax = "Fed = R$%7.2f (%.2f%%), Est = R$%7.2f (%.2f%%)" % (
        nacional_federal_value,
        nacional_federal_percent,
        estadual_value,
        estadual_percent
    )
    report.append(AposA8PrintLine(AposA8Align.CENTER, federal_tax, AposA8CommandType.PRINT_TEXT))

    law_cite = "(Conforme Lei Fed. 12.741/2012)"
    report.append(AposA8PrintLine(AposA8Align.CENTER, law_cite, AposA8CommandType.PRINT_TEXT))

    report.append(APOS_DASH)


def append_garena_coupons(mb_context, order, report):
    TK_DECRYPT_GARENA_COUPON = create_token("F", "99", "2")

    garena_custom_property = order.find("CustomOrderProperties/OrderProperty[@key='GARENA_COUPONS']")
    garena_coupons = None
    if garena_custom_property is not None:
        garena_coupons = garena_custom_property.get("value").split("|")
    if not garena_coupons:
        return

    request_data = []
    for coupon in garena_coupons:
        coupon_obj = literal_eval(coupon)
        coupon_id = coupon_obj['couponId']
        request_data.append(coupon_id)

    try:
        ret = mb_context.MB_EasySendMessage(
            "Garena",
            TK_DECRYPT_GARENA_COUPON,
            data="|".join(request_data),
            timeout=5000000
        )
        if ret.token == TK_SYS_NAK or not ret.data:
            raise VoidOrderException("Error decrypting garena coupon")
    except:
        return report

    garena_coupon_data = {}
    decrypted_coupons = json.loads(ret.data)
    for coupon in decrypted_coupons:
        coupon_data = json.loads(coupon['coupon_data'])
        garena_coupon_data.setdefault(coupon_data['promo']['description'], []).append(coupon['decrypted_data'])

    for promo in garena_coupon_data:
        promo_without_accents = remove_accents(promo)
        report.append(AposA8PrintLine(AposA8Align.CENTER, promo_without_accents, AposA8CommandType.PRINT_TEXT))

        for decrypted_coupon in garena_coupon_data[promo]:
            report.append(AposA8PrintLine(AposA8Align.CENTER, decrypted_coupon, AposA8CommandType.PRINT_TEXT))

        report.append(APOS_DASH)



# -*- coding: utf-8 -*-

import os
import cfgtools
import pyscripts

from msgbus import MBEasyContext
from messagehandler import MessageHandler
from eventhandler import CapptaEventHandler, CapptaCallback
from old_helper import config_logger, read_swconfig
from debug_helper import import_pydevd
from cappta import FakeCapptaProcessor, SynchCapptaProcessor

REQUIRED_SERVICES = "StoreWideConfig"
LOADER_CFG = os.environ["LOADERCFG"]


def main():
    import_pydevd(LOADER_CFG, 9122)

    config = cfgtools.read(LOADER_CFG)
    comp_no = int(config.find_value("CapptaConfig.PosId"))

    config_logger(LOADER_CFG, 'Cappta')
    mbcontext = MBEasyContext("Sitef%02d" % comp_no)
    pyscripts.mbcontext = mbcontext

    config = cfgtools.read(LOADER_CFG)
    is_fake = (config.find_value("CapptaConfig.IsFake") or "false").lower() == "true"
    env = int(config.find_value("CapptaConfig.Environment") or 2)
    module_directory = config.find_value("CapptaConfig.ModuleDirectory")

    pos_id = config.find_value("CapptaConfig.PosId")
    pdv_tef = config.find_value("CapptaConfig.PdvTef")

    try:
        pdv_tef = str(int(pdv_tef))
    except Exception as _:
        pdv_tef = pos_id

    electronic_payments = {1: True, 2: True, 3: False, 4: False}

    if config.find_group("SitefConfig.ElectronicTypes") is not None:
        for key in config.find_group("SitefConfig.ElectronicTypes").keys or []:
            try:
                enabled = (key.values[0] or "false").lower() == "true"
                electronic_payments[int(key.name)] = enabled
            except Exception: # noqa
                pass

    cnpj = config.find_value("CapptaConfig.CnpjLoja")
    auth_key = config.find_value("CapptaConfig.ChaveAutenticacao")

    if is_fake:
        cappta_processor = FakeCapptaProcessor()
    else:
        cappta_callback = CapptaCallback(mbcontext)
        cappta_processor = SynchCapptaProcessor(pos_id, pdv_tef, cnpj, auth_key, module_directory, cappta_callback.callback, env)

    cappta_event_handler = CapptaEventHandler(mbcontext, cappta_processor, electronic_payments)
    cappta_message_handler = MessageHandler(mbcontext, "Sitef%02d" % comp_no, "Cappta", REQUIRED_SERVICES, cappta_event_handler)

    cappta_message_handler.handle_events()



from abc import ABCMeta, abstractmethod

from externals.util import BaseThread


class DeviceServiceThread(BaseThread):
    __metaclass__ = ABCMeta

    @abstractmethod
    def run(self):
        # type: () -> None

        raise NotImplementedError()



from domain import SessionRepository
from old_helper import BaseRepository
from persistence import Connection  # noqa


class MwSessionRepository(SessionRepository, BaseRepository):
    def __init__(self, mbcontext):
        super(MwSessionRepository, self).__init__(mbcontext)

    def get_unique_user_session_count(self, initial_date, end_date):
        def inner_func(conn):
            # type: (Connection) -> int
            query = "select count(1) from UserSession where BusinessPeriod >= '{0}' and BusinessPeriod <= '{1}'"\
                .format(initial_date.strftime("%Y%m%d"),
                        end_date.strftime("%Y%m%d"))
            for line in conn.select(query):
                return int(line.get_entry(0))

        return self.execute_with_connection(inner_func)



from json import dumps
from typing import Dict, Tuple

from ._Action import Action
from ._Conditions import Conditions
from ._Exclusions import Exclusions


class Benefit(object):
    def __init__(self, raw_benefit):
        # type: (Dict) -> None

        self.id = raw_benefit.get("id")
        self.name = raw_benefit.get("name")
        self.user_description = raw_benefit.get("userDescription")
        self.priority = int(raw_benefit.get("priority")) if raw_benefit.get("priority") else None
        self.automatic = raw_benefit.get("automatic")
        self.tip_included = raw_benefit.get("tipIncluded") if raw_benefit.get("tipIncluded") else False
        self.ask_value = bool(raw_benefit.get("askValue"))
        conditions = raw_benefit.get("conditions") if raw_benefit.get("conditions") else {}
        self.conditions = Conditions(conditions)
        actions = [self._get_action(e) for e in raw_benefit.get("actions", {}).items()]
        self.actions = actions
        exclusions = raw_benefit.get("exclusions") if raw_benefit.get("exclusions") else {}
        self.exclusions = Exclusions(exclusions)
        self.raw = raw_benefit
        self.times_to_apply = raw_benefit.get("timesToApply")

    def to_json(self):
        # type: () -> str

        return dumps(self, default=lambda o: o.__dict__ if hasattr(o, '__dict__') else str(o), sort_keys=True)

    @staticmethod
    def _get_action(e):
        # type: (Tuple) -> Action

        return Action({"order": e[0], "name": e[1].items()[0][0], "value": e[1].items()[0][1]})



from _SatAposA8FiscalPrinter import SatAposA8FiscalPrinter
from _NfceAposA8FiscalPrinter import NfceAposA8FiscalPrinter
from _AposA8PrintLine import AposA8PrintLine
from _AposA8CommandType import AposA8CommandType
from _AposA8Align import AposA8Align
from _AposA8Constants import AposA8Constants

__all__ = ["SatAposA8FiscalPrinter", "NfceAposA8FiscalPrinter", "AposA8PrintLine", "AposA8CommandType", "AposA8Align",
           "AposA8Constants"]



from typing import List, Optional

import persistence
from msgbus import MBEasyContext
from pos_model import OrderKey, PosConnection
from _ModuleLogger import logger


class OrderRepository(object):
    def __init__(self, mbcontext, pos_list):
        # type: (MBEasyContext, list) -> None
        self.mbcontext = mbcontext
        self.pos_list = pos_list

    def get_all_orders_to_disable(self):
        # type: () -> List[OrderKey]

        void_state_id = 4
        orders_to_disable = []  # type: List[OrderKey]

        for pos_id in self.pos_list:  # type: int
            pos_connection = None  # type: Optional[PosConnection]

            try:
                pos_connection = persistence.Driver().open(self.mbcontext, dbname=str(pos_id))
                cursor = pos_connection.select("""
                    SELECT o.OrderId
                    FROM Orders o
                    INNER JOIN OrderCustomProperties ocp_include ON o.OrderId = ocp_include.OrderId
                    AND ocp_include.key = 'FISCAL_ID'
                    LEFT JOIN OrderCustomProperties ocp_exclude ON o.OrderId = ocp_exclude.OrderId
                    AND ocp_exclude.key in ('ORDER_DISABLED', 'CANCELED_AFTER_PAID')
                    WHERE ocp_exclude.OrderId IS NULL
                      AND o.StateId = {0}
                    ORDER BY o.OrderId DESC
                    LIMIT 100
                """.format(void_state_id))

                if cursor and cursor.rows() > 0:
                    for row in cursor:
                        order_id = row.get_entry(0)
                        orders_to_disable.append(OrderKey(pos_id=str(pos_id), order_id=order_id))

            except Exception as _:
                logger.exception("Error getting orders to disable from pos [{}]".format(pos_id))
            finally:
                if pos_connection:
                    pos_connection.close()

        return orders_to_disable

    def mark_order_disabled(self, pos_id, order_id):
        pos_connection = None
        try:
            pos_connection = persistence.Driver().open(self.mbcontext, dbname=str(pos_id))
            self._insert_order_disabled(pos_connection, order_id, "true")
        except Exception as _:
            logger.exception("Erro marcando order como disabled")
        finally:
            if pos_connection:
                pos_connection.close()

    def mark_order_canceled(self, pos_id, order_id):
        pos_connection = None
        try:
            pos_connection = persistence.Driver().open(self.mbcontext, dbname=str(pos_id))
            self._insert_order_canceled(pos_connection, order_id, "true")
        except Exception as _:
            logger.exception("Erro marcando order como cancelada")
        finally:
            if pos_connection:
                pos_connection.close()

    def mark_order_not_disabled(self, pos_id, order_id):
        pos_connection = None
        try:
            pos_connection = persistence.Driver().open(self.mbcontext, dbname=str(pos_id))
            self._insert_order_disabled(pos_connection, order_id, "false")
        except Exception as _:
            logger.exception("Erro marcando order como 'nao inutilizada'")
        finally:
            if pos_connection:
                pos_connection.close()

    @staticmethod
    def _insert_order_disabled(pos_connection, order_id, value):
        pos_connection.query("""INSERT INTO OrderCustomProperties (OrderId, Key, Value) VALUES ({0:s}, "ORDER_DISABLED", "{1:s}")""".format(order_id, value))

    @staticmethod
    def _insert_order_canceled(pos_connection, order_id, value):
        pos_connection.query("""INSERT INTO OrderCustomProperties (OrderId, Key, Value) VALUES ({0:s}, "CANCELED_AFTER_PAID", "{1:s}")""".format(order_id, value))



from .tz import *

__all__ = ["tzutc", "tzoffset", "tzlocal", "tzfile", "tzrange",
           "tzstr", "tzical", "tzwin", "tzwinlocal", "gettz",
           "enfold", "datetime_ambiguous", "datetime_exists"]



from production.box._OrderChangerProductionBox import OrderChangerProductionBox


class CFHHiderBox(OrderChangerProductionBox):
    def __init__(self, name, sons, cfh_items, logger=None):
        super(CFHHiderBox, self).__init__(name, sons, logger)
        self.cfh_items = cfh_items

    def change_order(self, order):
        order.items = self._find_and_remove_cfh_items(order.items[:])
        return order

    def _find_and_remove_cfh_items(self, items):
        new_items = []
        for item in items:
            if item.part_code in self.cfh_items:
                for son in item.items:
                    son.handle_remove_father(item)
                    new_items.extend(self._find_and_remove_cfh_items([son]))
            else:
                item.items = self._find_and_remove_cfh_items(item.items)
                new_items.append(item)

        return new_items



from _AppendFiscalFiles import AppendFiscalFilesProcessor
from _CleanDatabases import CleanDatabasesProcessor
from _Maintenance import MaintenanceProcessor
from _RecreateXMLFiles import RecreateXMLFilesProcessor
from _CleanJsonProcessors import CleanJsonProcessors


import json
import logging

import persistence

from datetime import datetime, timedelta
from threading import Lock

from sysactions import get_podtype, action, get_model, StopAction
from systools import sys_log_exception

from .. import mb_context
from ..util import get_ruptured_products_from_remote_order
from pyscriptscache import cache as _cache

_product_price = {}
product_by_barcode = {}
rupture_treated_events = []
products_cache = None
last_cache_time = None
lock = Lock()
last_snapshot_id = None

logger = logging.getLogger("PosActions")


@action
def getProducts(pos_id, force=False, snapshot_id=None):
    global products_cache
    global rupture_treated_events
    global last_cache_time
    global last_snapshot_id

    force = str(force).lower() == "true"
    seconds_to_renew_cache = 3600

    if lock.locked():
        raise StopAction()

    with lock:
        now = datetime.now()
        seconds_to_renew = _get_seconds_to_renew(seconds_to_renew_cache) if not force else 0
        reached_cache_time = _reached_seconds_to_renew_cache(now, seconds_to_renew)
        changed_last_snapshot_id = snapshot_id is not None and last_snapshot_id != snapshot_id
        if not products_cache or reached_cache_time or changed_last_snapshot_id:
            last_cache_time = now
            products_cache = _get_products(pos_id)

            if snapshot_id is not None:
                last_snapshot_id = snapshot_id

    return json.dumps(products_cache)


def _get_seconds_to_renew(seconds_to_renew_cache):
    seconds_to_renew = 180
    try:
        seconds_to_renew = int(seconds_to_renew_cache)
    except (TypeError, ValueError) as _:
        pass

    return seconds_to_renew


def _rupture_is_already_handled(rupture_event_id):
    return rupture_event_id is None or rupture_event_id in rupture_treated_events


def _reached_seconds_to_renew_cache(now, seconds_to_renew_cache):
    return not last_cache_time or now > last_cache_time + timedelta(seconds=seconds_to_renew_cache)


def _get_products(pos_id):
    global product_by_barcode
    product_by_barcode = {}

    query = """SELECT P.ProductCode,
                P.ProductName,
                PCP.CustomParamValue AS BarCode,
                NULLIF(Production.JITLines,'None') AS ProductionLine,
                CASE WHEN COALESCE(PT.Tag, PCP1.CustomParamValue) IS NULL THEN 'false' ELSE 'true' END AS CFH,
                PTY.TypeDescr as ProductType
                FROM Product P
                LEFT JOIN Production ON Production.ProductCode=P.ProductCode
                LEFT JOIN ProductCustomParams PCP ON PCP.ProductCode=P.ProductCode AND PCP.CustomParamId='BarCode'
                LEFT JOIN ProductCustomParams PCP1 ON PCP1.ProductCode=P.ProductCode AND PCP1.CustomParamId='CFH'
                AND PCP1.CustomParamValue = 'true'
                LEFT JOIN ProductTags PT ON PT.ProductCode=P.ProductCode AND PT.Tag='CFH=true'
                LEFT JOIN ProductKernelParams PKP ON PKP.ProductCode=P.ProductCode
                LEFT JOIN ProductType PTY ON PTY.TypeId=PKP.ProductType
                ORDER BY P.ProductName"""
    prices = get_prices(pos_id)
    prod_list = {}

    try:
        ruptured_items = get_ruptured_products_from_remote_order()
        conn = persistence.Driver().open(mb_context, dbname=str(pos_id))
        cursor = conn.select(query)
        for row in cursor:
            plu, name, barcode, prodline, cfh, prod_type = map(row.get_entry, ("ProductCode", "ProductName", "BarCode",
                                                                               "ProductionLine", "CFH", "ProductType"))
            product = {
                "plu": plu,
                "name": name,
                "price": prices.get(plu),
                "barcode": barcode,
                "prodline": prodline,
                "ruptured": plu in ruptured_items,
                "CFH": cfh,
                "type": prod_type
            }
            prod_list[plu] = product
            if barcode:
                try:
                    product_by_barcode[int(barcode)] = product
                except Exception as _:
                    logger.warning("Invalid barcode for product '{} ({})'".format(name, plu))
    except Exception as _:
        sys_log_exception("Error getting product list")
    return prod_list


@action
def getPrices(pos_id):
    return json.dumps(get_prices(pos_id))


def get_prices(pos_id):
    part_data = {}
    tag_data = {}
    conn = persistence.Driver().open(mb_context, dbname=str(pos_id))
    part_data = _get_part_data(conn)
    tag_data = _get_tag_data(conn)

    pod_type = get_podtype(get_model(pos_id))
    from table_actions import get_price_list_from_model
    price_list = get_price_list_from_model(pos_id, pod_type=pod_type)
    prices = _get_price_data(conn, price_list)

    def get_price_by_product_type_recursively(part_entries, product, context):
        context = context if context else "1"
        price = _get_price(product, context, pod_type, price_list)
        for part_params in part_entries:
            product_type = part_params["product_type"]
            part_code = part_params["part_code"]
            default_quantity = float(part_params["default_qty"] or 1)

            full_context = "{}.{}".format(product, part_code)
            if product_type == 0:  # product
                best_price = _get_price(part_code, product, pod_type, price_list)
                price += best_price * default_quantity
            elif product_type == 1:  # option
                default_part = tag_data.get(product.split('.')[-1], {}).get(part_code)
                if default_part:
                    best_price = None
                    for default in default_part:
                        default_price = _get_price(default, full_context, pod_type, price_list)
                        best_price = min(default_price, best_price) if best_price else default_price

                    price += best_price * default_quantity
                else:
                    option_prices = []
                    for part in part_params["product_codes"]:
                        if part in part_data:
                            part_price = get_price_by_product_type_recursively(part_data[part], part, full_context)
                        else:
                            part_price = _get_price(part, full_context, pod_type, price_list)

                        option_prices.append(part_price)

                    price += min(option_prices or 0) * default_quantity
            elif product_type == 2 and part_code in part_data:
                price += get_price_by_product_type_recursively(part_data[part_code], "{}.{}".format(full_context, part_code), context)
        return price

    for product, part_entries in part_data.iteritems():
        updated_prices = {}
        if product in prices:
            for context in prices[product]:
                updated_prices.update({context: get_price_by_product_type_recursively(part_entries, product, context)})
        else:
            updated_prices.update({None: get_price_by_product_type_recursively(part_entries, product, None)})

        prices[product] = updated_prices

    return prices


def _get_price(product, context, pod_type, price_list):
    best_price = _cache.get_best_price(context, product, pod_type, price_list)
    unit_price, add_price, _ = best_price if best_price else (0, 0, 0)
    return float(unit_price or add_price or 0)


def get_product_by_barcode(barcode):
    if barcode and int(barcode) in product_by_barcode:
        return product_by_barcode[int(barcode)]

    return None


def get_product_price_from_cache(plu, pos_id):
    if _product_price == {}:
        _get_products(pos_id)
    return _product_price[int(plu)]


def _get_part_data(conn):
    part_data = {}
    cursor = conn.select("""
        SELECT
            part.productCode,
            part.partcode,
            part.defaultQty,
            params.ProductType,
            class.ProductCodes
        FROM ProductPart part
        JOIN ProductKernelParams params
            ON params.ProductCode = part.PartCode
        LEFT JOIN (
            SELECT
                classcode,
                group_concat(ProductCode) as ProductCodes
            FROM ProductClassification
            GROUP BY ClassCode
        ) class
             ON class.ClassCode = part.PartCode
        WHERE part.MinQty > 0
        ORDER BY part.ProductCode;
        """)
    for row in cursor:
        part_data.setdefault(row.get_entry(0), []).append({
            "part_code": row.get_entry(1),
            "default_qty": row.get_entry(2),
            "product_type": int(row.get_entry(3)),
            "product_codes": (row.get_entry(4) or "").split(',')
        })
    return part_data


def _get_tag_data(conn):
    tag_data = {}
    cursor = conn.select("""
        SELECT CustomAttr, PartCode, ProductCode
        FROM ProductPart
        WHERE CustomAttr LIKE '%defaultOption%'
        """)
    for row in cursor:
        product_code = row.get_entry("ProductCode")
        try:
            part_code = row.get_entry("PartCode")
            default_options_json = json.loads(row.get_entry("CustomAttr"))
            default_options = default_options_json.get("defaultOption")
            if not default_options:
                continue

            if product_code not in tag_data:
                tag_data[product_code] = {}

            tag_data[product_code].update({part_code: default_options.split("|")})
        except Exception as _:
            logger.warning("Errro getting default options for product {}".format(product_code))

    return tag_data


def _get_price_data(conn, price_list=""):
    price_data = {}

    prices_list = price_list.split('.')[::-1]
    for price_list_id in prices_list:
        cursor = conn.select("""
            SELECT ProductCode, Context, CAST(COALESCE(DefaultUnitPrice, AddedUnitPrice, 0) AS FLOAT)
            FROM Price
            WHERE PriceListId = '{}' AND DATETIME('now') BETWEEN ValidFrom AND ValidThru;
            """.format(price_list_id))
        for row in cursor:
            product_code, context, price = row
            if product_code not in price_data:
                price_data[product_code] = {context: float(price)}
            elif context not in price_data[product_code]:
                price_data[product_code][context] = float(price)
    return price_data



# -*- coding: utf-8 -*-

class Event(object):
    # Evento que indica que uma nova mensagem do SAC está disponível
    SAC_CHAT_MESSAGE_NEW = "SacChatMessageNew"
    # Evento para informar o servidor que novas mensagens foram enviadas pela loja
    POS_CHAT_MESSAGE_NEW = "PosChatMessageNew"
    # Evento que o servidor envia para a loja para informar que as mensagens enviadas pela loja foram recebidas
    SAC_CHAT_MESSAGE_ACK = "SacChatMessageAck"
    # Evento que a loja envia para indicar que as mensagens do SAC foram recebidas com sucesso
    POS_CHAT_MESSAGE_ACK = "PosChatMessageAck"




def discount_in_all_items(benefit):
    actions = [str(e[1].items()[0][0]) for e in benefit.get("actions", {}).items()]
    actions_pass = ['discountPercentageOnTotal',
                    'discountAmountOnTotal',
                    'discountPercentageByGroupsOnTotal',
                    'discountAmountByGroupsOnTotal']

    has_item_pass = [item for item in actions if item in actions_pass]
    return len(has_item_pass) > 0


def check_item_in_action(benefit, part_code):
    items = [e[1].items()[0][1] for e in benefit.get("actions", {}).items()]
    items = [str(i[0]["item"]) for i in items]
    return len(items) > 0 and part_code in items




class Command(object):

    def __init__(self, raw_data):
        # type: (dict) -> None

        self.id = raw_data["id"]
        self.type = raw_data["type"]
        self.data = None

        if self.type == "ReprocessSales":
            self.data = raw_data["data"]["fromDate"]



from sysactions import send_message, check_operator_logged, get_model, action
from msgbus import MBException, TK_CDRAWER_OPEN


@action
def doOpenDrawer(pos_id, check_oper="true", *args):
    model = get_model(pos_id)
    if check_oper == "true":
        check_operator_logged(pos_id, model=model, can_be_blocked=False)

    drawers = model.findall("CashDrawer")
    for drawer in drawers:
        from threading import Thread
        Thread(target=thread_open_drawer, args=(drawer,)).start()


def thread_open_drawer(drawer):
    try:
        send_message(drawer.get("name"), TK_CDRAWER_OPEN)
    except MBException:
        pass



class IntervalConfiguration(object):
    def __init__(self, name, start_time, end_time):
        self.name = name
        self.start_time = start_time
        self.end_time = end_time
        self.orders = []



class RequiredItemOnSale(object):
    def __init__(self, obj):
        self.item_id = int(obj.get("item")) if obj.get("item") else None
        self.item_id = int(obj.get("item_id")) if obj.get("item_id") and self.item_id is None else self.item_id
        self.context = obj.get("context")
        self.quantity = int(obj.get("quantity")) if obj.get("quantity") else None



import cfgtools


class Modifier(object):
    def __init__(self, key):
        # type: (cfgtools.Key) -> Modifier
        self.name = ''
        self.description = 0

        self.parse_modifier(key)

    def parse_modifier(self, modifier_key):
        # type: (cfgtools.Key) -> ()
        self.name = modifier_key.name
        self.description = modifier_key.values[0]



from actions import mb_context
from bustoken import TK_REMOTE_ORDER_GET_DELIVERY_FEE_CODE
from msgbus import TK_SYS_ACK

delivery_fee_code = None


def get_delivery_fee_code():
    global delivery_fee_code

    if delivery_fee_code:
        return delivery_fee_code

    msg = mb_context.MB_EasySendMessage("RemoteOrder", TK_REMOTE_ORDER_GET_DELIVERY_FEE_CODE, timeout=3000000)
    if msg.token == TK_SYS_ACK:
        delivery_fee_code = msg.data

    return delivery_fee_code



"""
futurize: automatic conversion to clean 2/3 code using ``python-future``
======================================================================

Like Armin Ronacher's modernize.py, ``futurize`` attempts to produce clean
standard Python 3 code that runs on both Py2 and Py3.

One pass
--------

Use it like this on Python 2 code:

  $ futurize --verbose mypython2script.py

This will attempt to port the code to standard Py3 code that also
provides Py2 compatibility with the help of the right imports from
``future``.

To write changes to the files, use the -w flag.

Two stages
----------

The ``futurize`` script can also be called in two separate stages. First:

  $ futurize --stage1 mypython2script.py

This produces more modern Python 2 code that is not yet compatible with Python
3. The tests should still run and the diff should be uncontroversial to apply to
most Python projects that are willing to drop support for Python 2.5 and lower.

After this, the recommended approach is to explicitly mark all strings that must
be byte-strings with a b'' prefix and all text (unicode) strings with a u''
prefix, and then invoke the second stage of Python 2 to 2/3 conversion with::

  $ futurize --stage2 mypython2script.py

Stage 2 adds a dependency on ``future``. It converts most remaining Python
2-specific code to Python 3 code and adds appropriate imports from ``future``
to restore Py2 support.

The command above leaves all unadorned string literals as native strings
(byte-strings on Py2, unicode strings on Py3). If instead you would like all
unadorned string literals to be promoted to unicode, you can also pass this
flag:

  $ futurize --stage2 --unicode-literals mypython2script.py

This adds the declaration ``from __future__ import unicode_literals`` to the
top of each file, which implicitly declares all unadorned string literals to be
unicode strings (``unicode`` on Py2).

All imports
-----------

The --all-imports option forces adding all ``__future__`` imports,
``builtins`` imports, and standard library aliases, even if they don't
seem necessary for the current state of each module. (This can simplify
testing, and can reduce the need to think about Py2 compatibility when editing
the code further.)

"""

from __future__ import (absolute_import, print_function, unicode_literals)
import future.utils
from future import __version__

import sys
import logging
import optparse
import os

from lib2to3.main import main, warn, StdoutRefactoringTool
from lib2to3 import refactor

from libfuturize.fixes import (lib2to3_fix_names_stage1,
                               lib2to3_fix_names_stage2,
                               libfuturize_fix_names_stage1,
                               libfuturize_fix_names_stage2)

fixer_pkg = 'libfuturize.fixes'


def main(args=None):
    """Main program.

    Args:
        fixer_pkg: the name of a package where the fixers are located.
        args: optional; a list of command line arguments. If omitted,
              sys.argv[1:] is used.

    Returns a suggested exit status (0, 1, 2).
    """
    
    # Set up option parser
    parser = optparse.OptionParser(usage="futurize [options] file|dir ...")
    parser.add_option("-V", "--version", action="store_true",
                      help="Report the version number of futurize")
    parser.add_option("-a", "--all-imports", action="store_true",
                      help="Add all __future__ and future imports to each module")
    parser.add_option("-1", "--stage1", action="store_true",
                      help="Modernize Python 2 code only; no compatibility with Python 3 (or dependency on ``future``)")
    parser.add_option("-2", "--stage2", action="store_true",
                      help="Take modernized (stage1) code and add a dependency on ``future`` to provide Py3 compatibility.")
    parser.add_option("-0", "--both-stages", action="store_true",
                      help="Apply both stages 1 and 2")
    parser.add_option("-u", "--unicode-literals", action="store_true",
                      help="Add ``from __future__ import unicode_literals`` to implicitly convert all unadorned string literals '' into unicode strings")
    parser.add_option("-f", "--fix", action="append", default=[],
                      help="Each FIX specifies a transformation; default: all.\nEither use '-f division -f metaclass' etc. or use the fully-qualified module name: '-f lib2to3.fixes.fix_types -f libfuturize.fixes.fix_unicode_keep_u'")
    parser.add_option("-j", "--processes", action="store", default=1,
                      type="int", help="Run 2to3 concurrently")
    parser.add_option("-x", "--nofix", action="append", default=[],
                      help="Prevent a fixer from being run.")
    parser.add_option("-l", "--list-fixes", action="store_true",
                      help="List available transformations")
    parser.add_option("-p", "--print-function", action="store_true",
                      help="Modify the grammar so that print() is a function")
    parser.add_option("-v", "--verbose", action="store_true",
                      help="More verbose logging")
    parser.add_option("--no-diffs", action="store_true",
                      help="Don't show diffs of the refactoring")
    parser.add_option("-w", "--write", action="store_true",
                      help="Write back modified files")
    parser.add_option("-n", "--nobackups", action="store_true", default=False,
                      help="Don't write backups for modified files.")
    parser.add_option("-o", "--output-dir", action="store", type="str",
                      default="", help="Put output files in this directory "
                      "instead of overwriting the input files.  Requires -n. "
                      "For Python >= 2.7 only.")
    parser.add_option("-W", "--write-unchanged-files", action="store_true",
                      help="Also write files even if no changes were required"
                      " (useful with --output-dir); implies -w.")
    parser.add_option("--add-suffix", action="store", type="str", default="",
                      help="Append this string to all output filenames."
                      " Requires -n if non-empty. For Python >= 2.7 only."
                      "ex: --add-suffix='3' will generate .py3 files.")

    # Parse command line arguments
    flags = {}
    refactor_stdin = False
    options, args = parser.parse_args(args)

    if options.write_unchanged_files:
        flags["write_unchanged_files"] = True
        if not options.write:
            warn("--write-unchanged-files/-W implies -w.")
        options.write = True
    # If we allowed these, the original files would be renamed to backup names
    # but not replaced.
    if options.output_dir and not options.nobackups:
        parser.error("Can't use --output-dir/-o without -n.")
    if options.add_suffix and not options.nobackups:
        parser.error("Can't use --add-suffix without -n.")

    if not options.write and options.no_diffs:
        warn("not writing files and not printing diffs; that's not very useful")
    if not options.write and options.nobackups:
        parser.error("Can't use -n without -w")
    if "-" in args:
        refactor_stdin = True
        if options.write:
            print("Can't write to stdin.", file=sys.stderr)
            return 2
    # Is this ever necessary?
    if options.print_function:
        flags["print_function"] = True

    # Set up logging handler
    level = logging.DEBUG if options.verbose else logging.INFO
    logging.basicConfig(format='%(name)s: %(message)s', level=level)
    logger = logging.getLogger('libfuturize.main')

    if options.stage1 or options.stage2:
        assert options.both_stages is None
        options.both_stages = False
    else:
        options.both_stages = True

    avail_fixes = set()

    if options.stage1 or options.both_stages:
        avail_fixes.update(lib2to3_fix_names_stage1)
        avail_fixes.update(libfuturize_fix_names_stage1)
    if options.stage2 or options.both_stages:
        avail_fixes.update(lib2to3_fix_names_stage2)
        avail_fixes.update(libfuturize_fix_names_stage2)

    if options.unicode_literals:
        avail_fixes.add('libfuturize.fixes.fix_unicode_literals_import')

    if options.version:
        print(__version__)
        return 0
    if options.list_fixes:
        print("Available transformations for the -f/--fix option:")
        # for fixname in sorted(refactor.get_all_fix_names(fixer_pkg)):
        for fixname in sorted(avail_fixes):
            print(fixname)
        if not args:
            return 0
    if not args:
        print("At least one file or directory argument required.",
              file=sys.stderr)
        print("Use --help to show usage.", file=sys.stderr)
        return 2

    unwanted_fixes = set(fixer_pkg + ".fix_" + fix for fix in options.nofix)

    extra_fixes = set()
    if options.all_imports:
        if options.stage1:
            prefix = 'libfuturize.fixes.'
            extra_fixes.add(prefix +
                            'fix_add__future__imports_except_unicode_literals')
        else:
            # In case the user hasn't run stage1 for some reason:
            prefix = 'libpasteurize.fixes.'
            extra_fixes.add(prefix + 'fix_add_all__future__imports')
            extra_fixes.add(prefix + 'fix_add_future_standard_library_import')
            extra_fixes.add(prefix + 'fix_add_all_future_builtins')
    explicit = set()
    if options.fix:
        all_present = False
        for fix in options.fix:
            if fix == 'all':
                all_present = True
            else:
                if ".fix_" in fix:
                    explicit.add(fix)
                else:
                    # Infer the full module name for the fixer.
                    # First ensure that no names clash (e.g.
                    # lib2to3.fixes.fix_blah and libfuturize.fixes.fix_blah):
                    found = [f for f in avail_fixes
                             if f.endswith('fix_{0}'.format(fix))]
                    if len(found) > 1:
                        print("Ambiguous fixer name. Choose a fully qualified "
                              "module name instead from these:\n" +
                              "\n".join("  " + myf for myf in found),
                              file=sys.stderr)
                        return 2
                    elif len(found) == 0:
                        print("Unknown fixer. Use --list-fixes or -l for a list.",
                              file=sys.stderr)
                        return 2
                    explicit.add(found[0])
        if len(explicit & unwanted_fixes) > 0:
            print("Conflicting usage: the following fixers have been "
                  "simultaneously requested and disallowed:\n" +
                  "\n".join("  " + myf for myf in (explicit & unwanted_fixes)),
                  file=sys.stderr)
            return 2
        requested = avail_fixes.union(explicit) if all_present else explicit
    else:
        requested = avail_fixes.union(explicit)
    fixer_names = (requested | extra_fixes) - unwanted_fixes

    input_base_dir = os.path.commonprefix(args)
    if (input_base_dir and not input_base_dir.endswith(os.sep)
        and not os.path.isdir(input_base_dir)):
        # One or more similar names were passed, their directory is the base.
        # os.path.commonprefix() is ignorant of path elements, this corrects
        # for that weird API.
        input_base_dir = os.path.dirname(input_base_dir)
    if options.output_dir:
        input_base_dir = input_base_dir.rstrip(os.sep)
        logger.info('Output in %r will mirror the input directory %r layout.',
                    options.output_dir, input_base_dir)

    # Initialize the refactoring tool
    if future.utils.PY26:
        extra_kwargs = {}
    else:
        extra_kwargs = {
                        'append_suffix': options.add_suffix,
                        'output_dir': options.output_dir,
                        'input_base_dir': input_base_dir,
                       }

    rt = StdoutRefactoringTool(
            sorted(fixer_names), flags, sorted(explicit),
            options.nobackups, not options.no_diffs,
            **extra_kwargs)

    # Refactor all files and directories passed as arguments
    if not rt.errors:
        if refactor_stdin:
            rt.refactor_stdin()
        else:
            try:
                rt.refactor(args, options.write, None,
                            options.processes)
            except refactor.MultiprocessingUnsupported:
                assert options.processes > 1
                print("Sorry, -j isn't " \
                      "supported on this platform.", file=sys.stderr)
                return 1
        rt.summarize()

    # Return error status (0 if rt.errors is zero)
    return int(bool(rt.errors))



from abc import abstractmethod, ABCMeta
from typing import Iterable

from model.event_dispatch import EventDispatch


class EventDispatcherService(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def dispatch_event(self, event_to_dispatch):
        # type: (EventDispatch) -> None
        raise NotImplementedError()

    def dispatch_events(self, events_to_dispatch):
        # type: (Iterable[EventDispatch]) -> None
        for e in events_to_dispatch:
            self.dispatch_event(e)



from typing import List

from report import Presenter  # noqa
from basereport import SimpleReport

from .builders import TableReportHeader, TableReportBody, ReportBuilder
from .dto import TableReportDto  # noqa


class TableReportPresenter(Presenter):
    def __init__(self, print_by, hide_unpriced_items, pre_account_with_discount, pos_id):
        self.print_by = print_by
        self.hide_unpriced_items = hide_unpriced_items
        self.pre_account_with_discount = pre_account_with_discount
        self.pos_id = pos_id

    def present(self, dto):
        # type: (TableReportDto) -> SimpleReport

        pos_id = self.pos_id
        report_builders = [
            TableReportHeader(dto.header, self.pre_account_with_discount),
            TableReportBody(dto.body, self.print_by, self.hide_unpriced_items, self.pre_account_with_discount, pos_id)
        ]  # type: List[ReportBuilder]

        report = []
        for builder in report_builders:
            report.extend(builder.generate())

        return SimpleReport(report, 38)



from abc import abstractmethod, ABCMeta


class FiscalProcessor(object):
    __metaclass__ = ABCMeta

    def __init__(self):
        self.total_taxes = {}

    @abstractmethod
    def terminate(self):
        pass

    @abstractmethod
    def request_fiscal(self, posid, order, tenders, paf=False, fiscal_id=None):
        pass

    @abstractmethod
    def do_validation(self, get_days_to_expiration=None):
        pass




from mw_helper import ensure_iterable

from ._OrderChangerProductionBox import OrderChangerProductionBox


class ComboTaggedWhenAllHasTagBox(OrderChangerProductionBox):
    def __init__(self, name, sons, tags, tag_to_add, logger=None):
        super(ComboTaggedWhenAllHasTagBox, self).__init__(name, sons, logger)
        self.tags = ensure_iterable(tags)
        self.tag_to_add = tag_to_add

    def change_order(self, order):
        for item in order.items:
            if item.item_type == "COMBO" and self.all_sons_have_tags(item.items):
                item.add_tag(self.tag_to_add)

        return order

    def all_sons_have_tags(self, sons):
        for son in sons:
            if son.item_type != 'OPTION':
                if not son.has_some_tag(self.tags):
                    return False
            else:
                if not self.all_sons_have_tags(son.items):
                    return False
            
        return True



from abc import ABCMeta, abstractmethod
from typing import Optional

from nperipherals_core.model.device import Device


class ReaderDevice(Device):
    __metaclass__ = ABCMeta

    @abstractmethod
    def read(self):
        # type: () -> Optional[str]
        raise NotImplementedError()



import json
import logging

from sysactions import send_message
from msgbus import TK_SYS_ACK, FM_PARAM, MBMessage
from model.customexception import ErrorAddingBenefits, ErrorGettingBenefits
from bustoken import TK_NDISCOUNT_ADD_BENEFIT, TK_NDISCOUNT_REMOVE, TK_NDISCOUNT_GET_BENEFITS


class NDiscountInteractor(object):
    def __init__(self):
        # type: () -> None
        self.logger = logging.getLogger("PosActions")

    @staticmethod
    def add_benefit(pos_id, benefit_id, benefit):
        # type: (int, str, json) -> bool

        data = json.dumps(dict(posId=pos_id, benefitId=benefit_id, benefit=benefit))

        response = send_message("nDiscount", TK_NDISCOUNT_ADD_BENEFIT, FM_PARAM, data, timeout=30000000)
        if response.token == TK_SYS_ACK:
            return True

        raise ErrorAddingBenefits(response.data)

    @staticmethod
    def get_benefits(pos_id, only_visible):
        # type: (int, bool) -> json

        data = json.dumps(dict(posId=pos_id, only_visible=only_visible))

        response = send_message("nDiscount", TK_NDISCOUNT_GET_BENEFITS, FM_PARAM, data, timeout=30000000)
        if response.token == TK_SYS_ACK:
            return json.loads(response.data)

        raise ErrorGettingBenefits(response.data)

    @staticmethod
    def remove_benefits(pos_id, benefit_ids=None):
        # type: (str, array) -> MBMessage

        data = json.dumps(dict({"pos_id": pos_id, "benefit": benefit_ids}))
        return send_message("nDiscount", TK_NDISCOUNT_REMOVE, FM_PARAM, data, timeout=30000000)

    @staticmethod
    def remove_benefit(pos_id, benefit_id):
        # type: (str, int) -> MBMessage

        data = json.dumps(dict({"pos_id": pos_id, "benefit": benefit_id}))
        return send_message("nDiscount", TK_NDISCOUNT_REMOVE, FM_PARAM, data, timeout=30000000)

    @staticmethod
    def test_connection():
        # type: () -> bool
        try:
            data = json.dumps(dict())
            response = send_message("nDiscount", TK_NDISCOUNT_GET_BENEFITS, FM_PARAM, data, timeout=30000000)
            if response:
                return True
            return False
        except Exception as _:
            return False



def is_delivery_order(model):
    order = model.find("CurrentOrder/Order")
    if not order:
        return False
    
    return order.find("CustomOrderProperties/OrderProperty[@key='REMOTE_ORDER_ID']") is not None



class NavigationProduct(object):
    def __init__(self, nav_id, class_code, product_code, name, background_color):
        self.nav_id = nav_id
        self.class_code = class_code
        self.product_code = product_code
        self.name = name
        self.background_color = background_color



# coding=utf-8
import json
import os
import cfgtools
import sysactions
from msgbus import TK_SYS_ACK
from sysactions import action, show_listbox, translate_message, StopAction, show_messagebox, show_keyboard, get_model
from bustoken import TK_REMOTE_ORDER_GET_LOGISTIC_PARTNERS, TK_REMOTE_ORDER_SEARCH_LOGISTIC

from .. import logger, mb_context
from model.customexception import LogisticPartnerNotFound, LogisticSearchError

logistic_partners = {}


@action
def select_logistic_partner(pos_id, order_id):
    # type: (str, str) -> None

    select_partner = _ask_for_logistic_partner(pos_id)
    if select_partner:
        _search_logistic_partner(order_id, pos_id, select_partner)


def _search_logistic_partner(order_id, pos_id, select_partner):
    params = [select_partner, order_id]

    if _should_ask_for_deliveryman_name():
        if select_partner == 'default':
            deliveryman_data = sysactions.show_keyboard(pos_id, "$DELIVERY_MAN_NAME")
            if deliveryman_data is not None and deliveryman_data != "":
                params.append(deliveryman_data)
            else:
                show_messagebox(pos_id, "$INSERT_DELIVERY_MAN_NAME")
                return

    data = json.dumps("\0".join(params))
    msg = mb_context.MB_EasySendMessage("RemoteOrder", TK_REMOTE_ORDER_SEARCH_LOGISTIC, data=data, timeout=10000000)
    if msg.token == TK_SYS_ACK:
        show_messagebox(pos_id, "$SEARCHING_LOGISTIC_PARTNER")
    else:
        raise LogisticSearchError()


def _ask_for_logistic_partner(pos_id):
    model = get_model(pos_id)
    partners = _get_logistic_partners(model)
    listbox_options = partners.values()
    index = show_listbox(pos_id, listbox_options, message="$DELIVERYMEN")
    if index is None:
        raise StopAction()
    
    select_partner = partners.keys()[partners.values().index(listbox_options[index])]
    return select_partner


def _get_logistic_partners(model):
    global logistic_partners
    
    if not logistic_partners:
        logger.debug("Getting Logistic Partners from Remote Order")
        msg = mb_context.MB_EasySendMessage("RemoteOrder", TK_REMOTE_ORDER_GET_LOGISTIC_PARTNERS, timeout=10000000)
        if msg.token == TK_SYS_ACK:
            logger.debug("Logistic Partners from Remote Order: {}".format(msg.data))
            partners = json.loads(msg.data)
            if partners:
                for partner in partners:
                    partner_i18n = str("LOGISTIC_PARTNER_" + partner.upper())
                    logistic_partners[partner] = translate_message(model, partner_i18n)
        else:
            logger.debug("Error getting Logistic Partners from Remote Order")
                
    if not logistic_partners:
        raise LogisticPartnerNotFound()
    
    return logistic_partners


def _should_ask_for_deliveryman_name():
    config = cfgtools.read(os.environ["LOADERCFG"])
    value = config.find_value("Customizations.DeliveryConfigurations.AskForDeliveryManName", False)
    value = str(value).upper() == 'TRUE'
    return value



import json
import logging
import time
from threading import Lock
from xml.etree import cElementTree as etree

import bustoken
import pyscripts
from msgbus import FM_PARAM
from mw_helper import show_message_dialog
from sysactions import close_asynch_dialog, show_messagebox, get_used_service, get_model, show_any_dialog, \
    show_keyboard, show_listbox

TK_ELECTRONIC_PAYMENT_SEND_RESPONSE = bustoken.create_token("0", "0E", "5")
TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT = bustoken.create_token("0", "0E", "2")

mbcontext = pyscripts.mbcontext
_current_dialog_id = {}
_current_dialog_lock = Lock()


def _close_current_dialog(pos_id):
    dialog_id = None
    with _current_dialog_lock:
        if pos_id in _current_dialog_id:
            dialog_id = _current_dialog_id[pos_id]
            del _current_dialog_id[pos_id]

    if dialog_id is not None:
        logger.debug("will close dialog {}".format(dialog_id))
        close_asynch_dialog(pos_id, dialog_id)


def _store_current_dialog(pos_id, dialog_id):
    _close_current_dialog(pos_id)

    with _current_dialog_lock:
        _current_dialog_id[pos_id] = int(dialog_id)


def _get_current_dialog(pos_id):
    with _current_dialog_lock:
        return _current_dialog_id.get(pos_id, None)


def _is_electronic_payment(event_type):
    return event_type == "ELECTRONIC_PAYMENT"


def _send_message_response(pos_id, response):
    payment_service = get_used_service(get_model(pos_id), "electronicpayment")
    if not payment_service:
        return
    mbcontext.MB_EasySendMessage(payment_service, token=TK_ELECTRONIC_PAYMENT_SEND_RESPONSE,
                                 format=FM_PARAM, data="{}|{}".format(pos_id, response))


def _show_message(pos_id, message, title):
    _close_current_dialog(pos_id)
    _store_current_dialog(pos_id, show_message_dialog(pos_id, title, message, "NOCONFIRM", assync=True))


def _show_question(pos_id, message, title, buttons):
    _close_current_dialog(pos_id)
    _send_message_response(pos_id, show_messagebox(pos_id, message, title, buttons=buttons))


def _show_barcode(pos_id, code_data, title, code_type):
    _close_current_dialog(pos_id)
    logger.debug("will show barcode")
    _store_current_dialog(pos_id, show_any_dialog(
        pos_id, "barcode", code_data, title, "", "$CANCEL",
        code_type, 60000, "", "", "", "", "", True))
    logger.debug("barcode is shown")


def _show_input(pos_id, message, title, input_type):
    _close_current_dialog(pos_id)
    if not input_type or input_type == "none":
        _send_message_response(pos_id, show_messagebox(pos_id, message, title, buttons="$OK"))
    else:
        _send_message_response(pos_id, show_keyboard(pos_id, message, title, numpad=input_type == "number"))


def _show_choices(pos_id, message, title, choices):
    _close_current_dialog(pos_id)
    choices_dict = json.loads(choices)
    choices_keys = sorted(choices_dict.keys())
    choices_values = [choices_dict[i] for i in choices_keys]

    index = show_listbox(pos_id, choices_values, message=message, title=title,
                         buttons="$OK|$CANCEL", icon="info", timeout=720000)
    if index is None:
        _cancel_current_payment(pos_id)
    _send_message_response(pos_id, choices_keys[index] if index is not None else "")


def _show_numpad(pos_id, message, title):
    _close_current_dialog(pos_id)
    value = show_keyboard(pos_id, message, title=title, mask="INTEGER", numpad=True)
    if value is None:
        _cancel_current_payment(pos_id)
    _send_message_response(pos_id, value if value is not None else "")


def _get_posid_from_data(data):
    return data.split('\\0')[0]


message_lock_dict = Lock()
message_lock = {}


def _get_pos_lock(data):
    with message_lock_dict:
        pos_id = _get_posid_from_data(data)
        if pos_id not in message_lock:
            message_lock[pos_id] = Lock()
        return message_lock[pos_id]


logger = logging.getLogger("PosActions")


def _parse_event_and_call(func):
    def inner_func(params):
        data, subject, event_type = params[:3]
        if not _is_electronic_payment(event_type):
            return

        with _get_pos_lock(data):
            logger.debug("electronic event: {} data {}".format(func, data))
            return func(*data.split('\\0'))

    return inner_func


def _dialog_resp_received(params):
    xml = params[0]
    dialog = etree.XML(xml).find("Dialog")
    dlg_id = int(dialog.get("id"))

    pos_id = None
    with _current_dialog_lock:
        for pos, dlg in _current_dialog_id.iteritems():
            if dlg == dlg_id:
                pos_id = pos
                break

    if not pos_id:
        return

    _cancel_current_payment(pos_id)
    _close_current_dialog(pos_id)


def _cancel_current_payment(pos_id):
    payment_service = get_used_service(get_model(pos_id), "electronicpayment")
    if not payment_service:
        return
    mbcontext.MB_EasySendMessage(payment_service, token=TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT)


def _close_dialogs(pos_id):
    _close_current_dialog(pos_id)


def init():
    pyscripts.subscribe_event_listener("SHOW_MESSAGE", _parse_event_and_call(_show_message))
    pyscripts.subscribe_event_listener("SHOW_QUESTION", _parse_event_and_call(_show_question))
    pyscripts.subscribe_event_listener("SHOW_BARCODE", _parse_event_and_call(_show_barcode))
    pyscripts.subscribe_event_listener("SHOW_INPUT", _parse_event_and_call(_show_input))
    pyscripts.subscribe_event_listener("SHOW_CHOICES", _parse_event_and_call(_show_choices))
    pyscripts.subscribe_event_listener("SHOW_NUMPAD", _parse_event_and_call(_show_numpad))
    pyscripts.subscribe_event_listener("CLOSE_DIALOGS", _parse_event_and_call(_close_dialogs))
    pyscripts.subscribe_event_listener("DIALOG_RESP", _dialog_resp_received)



from nperipherals_core.model.device import CashDrawer


class CashDrawerStatusInteractor(object):
    def __init__(self, device):
        # type: (CashDrawer) -> None
        self.device = device

    def get_status(self):
        connected = False
        try:
            self.device.initialize()
            self.device.connect()
            connected = True
            return self.device.get_drawer_status()

        finally:
            if connected:
                self.device.disconnect()



from __future__ import absolute_import
import sys

assert sys.version_info[0] < 3

from BaseHTTPServer import *
from CGIHTTPServer import *
from SimpleHTTPServer import *
try:
    from CGIHTTPServer import _url_collapse_path     # needed for a test
except ImportError:
    try:
        # Python 2.7.0 to 2.7.3
        from CGIHTTPServer import (
            _url_collapse_path_split as _url_collapse_path)
    except ImportError:
        # Doesn't exist on Python 2.6.x. Ignore it.
        pass



import json

import bustoken
import sysactions
from msgbus import TK_SYS_ACK

from .. import logger, mb_context

TK_REMOTE_ORDER_GET_STORE_STATUS = bustoken.create_token(bustoken.MSGPRT_LOW, "37", "214")


@sysactions.action
def get_remote_order_status(_=None):
    default_response = json.dumps(dict(isOnline=False,
                                       isOpened=False,
                                       lastExternalContact=None,
                                       closedSince=None))
    try:
        msg = mb_context.MB_EasySendMessage("RemoteOrder", TK_REMOTE_ORDER_GET_STORE_STATUS, timeout=5 * 1000000)
        if msg.token == TK_SYS_ACK:
            return msg.data
        raise Exception("Could not retrieve remote order status")
    except Exception as e:
        logger.error(e)
        return default_response



import time
from logging import getLogger

from nperipherals_core.model.custom_exceptions import FailedWritingToDevice
from nperipherals_core.model.device import WriterDevice, ReaderDevice
from typing import Union

logger = getLogger(__name__)


class PrintInteractor(object):
    def __init__(self, writer_device):
        # type: (Union[ReaderDevice, WriterDevice]) -> None

        self.writer_device = writer_device

    def print_data(self, data):
        # type: (str) -> None

        device_name = self.writer_device.get_name()
        write_retries = self.writer_device.get_write_retries()
        write_retry_sleep_in_seconds = self.writer_device.get_write_retry_sleep_in_seconds()

        logger.info("Writing to device [{}]".format(device_name))
        logger.debug("Writing [{}] to device [{}]".format(data, device_name))

        error_messages = []

        for i in range(write_retries):
            connected = False
            try:
                self.writer_device.initialize()
                self.writer_device.connect()
                connected = True
                self.writer_device.write(data)
                return

            except Exception as ex:
                error_messages.append("Exception #[{}]: [{}]".format(i, repr(ex)))
                logger.warning("Error writing to device [{}]".format(device_name), exc_info=True)
                time.sleep(write_retry_sleep_in_seconds)

            finally:
                if connected:
                    self.writer_device.disconnect()

        raise FailedWritingToDevice("|".join(error_messages))



# -*- coding: utf-8 -*-
from typing import List, Any, Tuple

import constants
import printer_wrapper

from datetime import datetime, timedelta
from multiprocessing import Lock
from msgbus import MBEasyContext
from old_helper import read_swconfig
from helper import remove_accents

lock = Lock()
cache_header = ""
last_cache_time_larger_paper = None
last_cache_time_small_paper = None


class ClicheHelper(object):
    def __init__(self, mb_context, cnpj_contribuinte, insc_estadual):
        # type: (MBEasyContext, str, str) -> None
        self.mb_context = mb_context
        self.cnpj_contribuinte = cnpj_contribuinte
        self.insc_estadual = insc_estadual
        self.cliche = None

    def get_cliche(self):
        if self.cliche is None:
            razao_social = read_swconfig(self.mb_context, "Store.RazaoSocial")
            end_logradouro = read_swconfig(self.mb_context, "Store.EndLogradouro")
            end_numero = read_swconfig(self.mb_context, "Store.EndNumero").strip() or "S/N"
            end_compl = (read_swconfig(self.mb_context, "Store.EndCompl") or "").strip()
            bairro = read_swconfig(self.mb_context, "Store.Bairro")
            municipio = read_swconfig(self.mb_context, "Store.Municipio")
            cep = read_swconfig(self.mb_context, "Store.CEP")
            uf = read_swconfig(self.mb_context, "Store.UF")
            cnpj_contribuinte = self.cnpj_contribuinte
            inscr_estadual = self.insc_estadual
            formatted_ie = "IE: %s" % inscr_estadual
            self.cliche = [
                razao_social,
                "%s, %s %s %s" % (end_logradouro, end_numero, end_compl, bairro),
                "%s, %s %s-%s" % (municipio, uf, cep[:5], cep[5:]),
                "CNPJ: %s.%s.%s/%s-%s %s" % (cnpj_contribuinte[:2],
                                             cnpj_contribuinte[2:5],
                                             cnpj_contribuinte[5:8],
                                             cnpj_contribuinte[8:12],
                                             cnpj_contribuinte[12:],
                                             formatted_ie)
            ]
        return self.cliche

    def get_header_coupon(self, is_larger_paper=True):
        global cache_header

        last_cache_time = self.get_last_cache_time(is_larger_paper)
        seconds_to_renew = self.get_seconds_to_renew_cache((60 * 60) * 8)
        if self.reached_seconds_to_renew_cache(last_cache_time, seconds_to_renew):
            with lock:
                self.set_last_cache_time(is_larger_paper)
                cache_header = ""
                cache_header += constants.TXT_ALIGN_LT
                logo = printer_wrapper.get_logo(is_larger_paper)
                lines = self.get_cliche()

                if logo is None:
                    cache_header = constants.TXT_ALIGN_CT
                    for line in lines:
                        cache_header += remove_accents(line) + "\n"
                    return cache_header

                if not is_larger_paper:
                    cache_header = constants.TXT_ALIGN_CT
                    cache_header += logo
                    cache_header += constants.TXT_ALIGN_CT
                    for line in lines:
                        cache_header += remove_accents(line) + "\n"
                    return cache_header

                lines = self._adjust_size_lines_address(lines)
                logo = logo.split("\n")
                for line_logo, line_address in self._zip(logo, lines):
                    cache_header += "%s %s\n" % (line_logo.ljust(13, " "), remove_accents(line_address))

        return cache_header

    def _adjust_size_lines_address(self, lines):
        # type: (List[str, Any]) -> List[str]

        new_lines = []
        line_max_size = 44

        for line in lines:
            if len(line) < line_max_size:
                new_lines.append(line)
                continue

            new_lines += self.manager_lines_size(line, line_max_size)

        return new_lines

    def _zip(self, list_one, list_two):
        # type: (List[str], List[str]) -> List[Tuple]

        max_size = len(list_one) if len(list_one) > len(list_two) else len(list_two)
        columns = []
        for index in range(max_size):
            columns.append((self._get_column(index, list_one), self._get_column(index, list_two)))

        return columns

    @staticmethod
    def _get_column(index, list_data):
        # type: (int, List[str]) -> str

        try:
            return list_data[index]
        except IndexError:
            return ""


    @staticmethod
    def manager_lines_size(line, max_size=35):
        # type: (str, int) -> list

        lines = []
        while line != '':
            current_line = line[0:max_size]
            lines.append(current_line)
            line = line[len(current_line):]

        return lines

    @staticmethod
    def get_last_cache_time(is_larger_paper):
        global last_cache_time_larger_paper
        global last_cache_time_small_paper

        return last_cache_time_larger_paper if is_larger_paper else last_cache_time_small_paper

    @staticmethod
    def set_last_cache_time(is_larger_paper):
        global last_cache_time_larger_paper
        global last_cache_time_small_paper
        if is_larger_paper:
            last_cache_time_larger_paper = datetime.now()
            return

        last_cache_time_small_paper = datetime.now()

    @staticmethod
    def get_seconds_to_renew_cache(seconds_to_renew_cache):
        seconds_to_renew = 60 * 5
        try:
            seconds_to_renew = int(seconds_to_renew_cache)
        except TypeError as _:
            pass
        return seconds_to_renew

    @staticmethod
    def reached_seconds_to_renew_cache(last_cache, seconds_to_renew_cache):
        return not last_cache or datetime.now() > last_cache + timedelta(seconds=seconds_to_renew_cache)



# -*- coding: utf-8 -*-
import sysactions
import logging

from posot import OrderTakerException
from custom_sysactions import user_authorized_to_execute_action, block_action_if_order_items_not_resolved

from actions.discount.clean_discounts import clean_order_payments
from actions.util import is_delivery_order
from actions.delivery.save_delivery_order import save_delivery_order

logger = logging.getLogger("PosActions")


@sysactions.action
@user_authorized_to_execute_action
@block_action_if_order_items_not_resolved
def doStoreOrder(pos_id, totalize="none", show_popup=True, *args):
    from posactions import doTotal, fill_customer_properties, customer_info_config
    model = sysactions.get_model(pos_id)
    sysactions.check_current_order(pos_id, model=model, need_order=True)

    pos_ot = sysactions.get_posot(model)
    xml_order = sysactions.get_current_order(model)
    if not clean_order_payments(pos_id, pos_ot, xml_order, True, True):
        return

    if sysactions.get_current_order(model).get("type") != "SALE":
        sysactions.show_messagebox(pos_id, message="$THIS_ORDER_TYPE_CANNOT_BE_STORED", icon="warning")
        return

    pod_function = sysactions.get_posfunction(model) if sysactions.get_podtype(model) in ("DT", "FC") else sysactions.get_podtype(model)
    if totalize == "none" and pod_function == "OT":
        totalize = "true"

    if (totalize.lower() != "true" or pod_function == "OT") and customer_info_config is not None:
        customer_document_config = customer_info_config[int(pos_id)]['document']
        customer_name_config = customer_info_config[int(pos_id)]['name']
        request_customer_info = "onSave" in customer_document_config or "onSave" in customer_name_config
        if request_customer_info:
            get_document = "onSave" in customer_document_config
            get_name = "onSave" in customer_name_config
            fill_customer_properties(model, pod_function, pos_id, pos_ot, get_doc=get_document, get_name=get_name)

    if totalize.lower() == "true":
        if doTotal(pos_id, screen_number="") == "Error":
            return

    order = model.find("CurrentOrder/Order")
    # Pre-venda
    pre_venda = None
    if pod_function == "OT":
        for prop in order.findall("CustomOrderProperties/OrderProperty"):
            key, value = prop.get("key"), prop.get("value")
            if key == "PRE_VENDA":
                pre_venda = value
                break
    try:
        pos_ot.storeOrder(pos_id)

        if pre_venda:
            sysactions.show_messagebox(pos_id, "Pre Venda: PV%010d" % int(pre_venda))
        elif _save_order_on_remote_order(order, pos_id, pos_ot):
            if show_popup:
                sysactions.show_messagebox(pos_id, "$ORDER_SAVED")
        else:
            sysactions.show_messagebox(pos_id, "$ERROR_SAVING_ORDER")
    except OrderTakerException as ex:
        sysactions.show_messagebox(pos_id, message="$ERROR_CODE_INFO|%d|%s" % (ex.getErrorCode(), ex.getErrorDescr()))

    order = model.find("CurrentOrder/Order")
    order_id = order.get("orderId")
    logger.debug("Pedido Salvo - Order %s - PosId %s" % (order_id, pos_id))


def _save_order_on_remote_order(order, pos_id, pos_ot):
    model = sysactions.get_model(pos_id)
    if is_delivery_order(model):
        order_id = order.get("orderId")

        try:
            save_delivery_order(order_id)
        except Exception as _:
            session = sysactions.get_operator_session(model)
            pos_ot.recallOrder(int(pos_id), int(order_id), session)
            return False

    return True



# -*- coding: utf-8 -*-

from msgbus import MBEasyContext
from old_helper import BaseRepository
from persistence import Connection
from typing import List

from ..model import OrderItem


class ProductRepository(BaseRepository):

    def __init__(self, mbcontext):
        # type: (MBEasyContext) -> None
        super(ProductRepository, self).__init__(mbcontext)

    def get_order_items_names(self, order_items):
        # type: (List[OrderItem]) -> List[OrderItem]
        def inner_func(conn):
            # type: (Connection) -> List[OrderItem]
            query = self._ProductNamesQuery
            pcodes_list = map(lambda order_item: order_item.pcode, order_items)
            pcodes_list = ','.join(map(str, pcodes_list))
            query = query.format(pcodes_list)
            product_names = [(int(x.get_entry(0)), x.get_entry(1)) for x in conn.select(query)]
            for product_tuple in product_names:
                for order_item in order_items:
                    if product_tuple[0] == order_item.pcode:
                        order_item.name = product_tuple[1]
            return order_items
        return self.execute_with_connection(inner_func)

    _ProductNamesQuery = """\
SELECT *
FROM Product
WHERE ProductCode IN ({0})
"""


class ChangeProdStateRequest(object):
    def __init__(self, order_id, view, state):
        # type: (str, str, str) -> None

        self.order_id = order_id
        self.view = view
        self.state = state



# -*- coding: utf-8 -*-
from cache_functions import UsersInfoCache
from logging import Logger

from messagebus import TokenCreator, TokenPriority, MessageBus
from messagehandler import MessageHandlerBuilder
from messageprocessor import \
    MessageProcessorMessageHandler, \
    DefaultMessageProcessorExecutorFactory, \
    LoggingProcessorCallback
from production.box import ViewBox

from production.manager import ProductionManager
from production.processor import OrderModifiedProcessor, GetOrderMaxPrepTimeProcessor, GetOrderXmlProcessor, \
    OrderModifiedIgnoredTypesProcessor, StatisticUpdateProcessor, PurgeOrdersProcessor, GetPrintersInfoProcessor
from production.processor._GetCoursesProcessor import GetCoursesProcessor
from production.processor.command import SetOrderStateProcessor, ToggleTagLineProcessor, UndoProcessor, \
    RefreshViewProcessor, EnablePathProcessor, DisablePathProcessor
from production.repository import ProductRepository, ProductionRepository
from production.treebuilder import ViewConfiguration, TreeBuilder, TreeConfig
from production.treebuilder.boxfactory import ProductionBoxFactory
from production.treebuilder.viewfactory import ViewFactory
from typing import List, Dict, Union

production_group = "C"
TK_PROD_REFRESH_VIEW = TokenCreator.create_token(TokenPriority.low, production_group, "1")
TK_PROD_SET_ORDER_STATE = TokenCreator.create_token(TokenPriority.low, production_group, "2")
TK_PROD_GET_ORDER_MAX_PREP_TIME = TokenCreator.create_token(TokenPriority.low, production_group, "3")
TK_PROD_GET_ORDER_XML = TokenCreator.create_token(TokenPriority.low, production_group, "4")  # 12582916
TK_PROD_UNDO = TokenCreator.create_token(TokenPriority.low, production_group, "7")
TK_PROD_TOGGLE_TAG_LINE = TokenCreator.create_token(TokenPriority.low, production_group, "8")

TK_PROD_DISABLE_PATH = TokenCreator.create_token(TokenPriority.low, production_group, "101")
TK_PROD_ENABLE_PATH = TokenCreator.create_token(TokenPriority.low, production_group, "100")
TK_PROD_GET_STATISTICS = TokenCreator.create_token(TokenPriority.low, production_group, "102")
TK_PROD_PURGE = TokenCreator.create_token(TokenPriority.low, production_group, "103")

TK_PROD_GET_PRINTERS_INFO = TokenCreator.create_token(TokenPriority.low, production_group, "201")
TK_PROD_GET_COURSES = TokenCreator.create_token(TokenPriority.low, production_group, "202")


class ProductionMessageHandlerBuilder(MessageHandlerBuilder):
    def __init__(
            self,
            message_bus,                # type: MessageBus
            purge_interval,             # type: int
            orders_life_time,           # type: int
            product_repository,         # type: ProductRepository
            production_repository,      # type: ProductionRepository
            manager_log_level,          # type: str
            audit_logger_prod_states,   # type: List
            boxes_config,               # type: List[TreeConfig]
            branches_config,            # type: List[TreeConfig]
            views_config,               # type: List[ViewConfiguration]
            kds_manager,                # type: str
            loggers                     # type: Dict[unicode, Logger]
    ):
        # type: (...) -> None

        self.message_bus = message_bus
        self.purge_interval = purge_interval
        self.orders_life_time = orders_life_time
        self.product_repository = product_repository
        self.production_repository = production_repository
        self.manager_log_level = manager_log_level
        self.audit_logger_prod_states = audit_logger_prod_states
        self.boxes_config = boxes_config
        self.branches_config = branches_config
        self.views_config = views_config
        self.kds_manager = kds_manager
        self.loggers = loggers

        self.production_manager = None  # type: Union[None, ProductionManager]
        self.message_handler = None
        self.user_cache_thread = None

    def build_singletons(self):
        if self.manager_log_level in self.loggers:
            manager_logger = self.loggers[self.manager_log_level]
        elif "ERROR" in self.loggers:
            manager_logger = self.loggers["ERROR"]
        elif len(self.loggers) > 0:
            manager_logger = self.loggers[self.loggers.keys()[0]]
        else:
            manager_logger = None

        self.production_manager = ProductionManager(
            self.production_repository,
            self.purge_interval,
            self.orders_life_time,
            self.message_bus,
            self.audit_logger_prod_states,
            self.kds_manager,
            manager_logger
        )

        production_box_factory = ProductionBoxFactory(
            self.production_manager,
            self.production_manager,
            self.production_manager,
            self.production_manager,
            self.product_repository,
            self.loggers
        )

        view_factory = ViewFactory(self.message_bus, self.loggers, self.production_manager, self.kds_manager)
        tree_builder = TreeBuilder(production_box_factory, view_factory)
        trees = tree_builder.build_trees(self.boxes_config, self.branches_config, self.views_config)

        root_boxes = []
        view_boxes = []
        for tree in trees:
            root_boxes.append(tree.root)
            leaves = [t for t in tree.leaves if isinstance(t, ViewBox)]
            view_boxes.extend(leaves)

        self.production_manager.set_root_boxes(root_boxes)
        self.production_manager.set_views(view_boxes)
        self.production_manager.initialize()

        self.user_cache_thread = UsersInfoCache(self.message_bus, manager_logger)
        self.user_cache_thread.daemon = True
        self.user_cache_thread.start()

        order_modified_processor = OrderModifiedProcessor(
            self.production_manager,
            self.product_repository,
            self.user_cache_thread
        )

        order_modified_ignored_types_processor = OrderModifiedIgnoredTypesProcessor()
        event_processors = {
            "ORDER_MODIFIED": order_modified_processor,
            "ORDER_MODIFIED_ORDER_PROPERTIES_CHANGED": order_modified_ignored_types_processor,
            "ORDER_MODIFIED_DO_TENDER": order_modified_ignored_types_processor,
            "ORDER_MODIFIED_DISCOUNT_APPLIED": order_modified_ignored_types_processor,
            "ORDER_MODIFIED_DISCOUNT_REMOVED": order_modified_ignored_types_processor,
            "ORDER_MODIFIED_CLEAR_TENDERS": order_modified_ignored_types_processor,
        }
        max_prep_time_processor = GetOrderMaxPrepTimeProcessor(self.production_manager, self.product_repository)

        message_processors = {
            TK_PROD_SET_ORDER_STATE: SetOrderStateProcessor(view_boxes),
            TK_PROD_TOGGLE_TAG_LINE: ToggleTagLineProcessor(view_boxes, self.loggers),
            TK_PROD_UNDO: UndoProcessor(view_boxes),
            TK_PROD_REFRESH_VIEW: RefreshViewProcessor(view_boxes),
            TK_PROD_ENABLE_PATH: EnablePathProcessor(self.production_manager),
            TK_PROD_DISABLE_PATH: DisablePathProcessor(self.production_manager),
            TK_PROD_GET_ORDER_MAX_PREP_TIME: max_prep_time_processor,
            TK_PROD_GET_ORDER_XML: GetOrderXmlProcessor(self.production_manager),
            TK_PROD_GET_STATISTICS: StatisticUpdateProcessor(self.production_manager),
            TK_PROD_PURGE: PurgeOrdersProcessor(self.production_manager),
            TK_PROD_GET_PRINTERS_INFO: GetPrintersInfoProcessor(view_boxes),
            TK_PROD_GET_COURSES: GetCoursesProcessor(self.product_repository)
        }

        callbacks = [LoggingProcessorCallback(manager_logger)]

        self.message_handler = MessageProcessorMessageHandler(
            event_processors,
            message_processors,
            None,
            DefaultMessageProcessorExecutorFactory(callbacks),
            manager_logger
        )

    def build_message_handler(self):
        return self.message_handler

    def destroy_message_handler(self, message_handler):
        # no need to destroy anything
        pass

    def destroy_singletons(self):
        self.production_manager.stop()
        self.user_cache_thread.stop()
        self.production_repository.stop()



from _FiscalParameterController import FiscalParameterController, FiscalParameterNotFound
from _InfAdicBuilderCommon import InfAdicBuilderCommon

__all__ = ["FiscalParameterController", "FiscalParameterNotFound", "InfAdicBuilderCommon"]



from production.model import ReportNameNotFoundException, PrinterNameFoundException
from production.treebuilder import ViewConfiguration
from ._CommonViewBuilder import get_extra, get_logger

from production.view import PrinterView


class PrinterViewBuilder(object):
    def __init__(self, message_bus, loggers):
        self.message_bus = message_bus
        self.loggers = loggers

    def build(self, config):
        # type: (ViewConfiguration) -> PrinterView

        if "ReportName" not in config.extra_config:
            raise ReportNameNotFoundException("PrinterView without report: {}".format(config.name))

        if "PrinterName" not in config.extra_config:
            raise PrinterNameFoundException("PrinterView without printer: {}".format(config.name))

        return PrinterView(config.name,
                           self.message_bus,
                           config.extra_config["ReportName"].encode("utf-8"),
                           config.extra_config["PrinterName"].encode("utf-8"),
                           config.extra_config.get("PickListName", "").encode("utf-8"),
                           int(get_extra(config, "Timeout", 10000)) * 1000,
                           get_extra(config, "TagOnPrint", "printed").split(";"),
                           get_logger(config, self.loggers))



from ._RealClock import RealClock
from ._MwOperatorNameRetriever import MwOperatorNameRetriever
from ._MwStoreRetriever import MwStoreRetriever
from ._MwBusinessPeriodOrderRepository import MwBusinessPeriodOrderRepository
from ._MwBusinessPeriodTransferRepository import MwBusinessPeriodTransferRepository
from ._MwSessionRepository import MwSessionRepository
from ._MsgBusTableService import MsgBusTableService
from ._MwTableOrderRetriever import MwTableOrderRetriever
from ._MwTipRepository import MwTipRepository

__all__ = [
    "RealClock",
    "MwOperatorNameRetriever",
    "MwStoreRetriever",
    "MwBusinessPeriodOrderRepository",
    "MwBusinessPeriodTransferRepository",
    "MwSessionRepository",
    "MsgBusTableService",
    "MwTableOrderRetriever",
    "MwTipRepository"
]



'''
This module provides a newnext() function in Python 2 that mimics the
behaviour of ``next()`` in Python 3, falling back to Python 2's behaviour for
compatibility if this fails.

``newnext(iterator)`` calls the iterator's ``__next__()`` method if it exists. If this
doesn't exist, it falls back to calling a ``next()`` method.

For example:

    >>> class Odds(object):
    ...     def __init__(self, start=1):
    ...         self.value = start - 2
    ...     def __next__(self):                 # note the Py3 interface
    ...         self.value += 2
    ...         return self.value
    ...     def __iter__(self):
    ...         return self
    ...
    >>> iterator = Odds()
    >>> next(iterator)
    1
    >>> next(iterator)
    3

If you are defining your own custom iterator class as above, it is preferable
to explicitly decorate the class with the @implements_iterator decorator from
``future.utils`` as follows:

    >>> @implements_iterator
    ... class Odds(object):
    ...     # etc
    ...     pass

This next() function is primarily for consuming iterators defined in Python 3
code elsewhere that we would like to run on Python 2 or 3.
'''

_builtin_next = next

_SENTINEL = object()

def newnext(iterator, default=_SENTINEL):
    """
    next(iterator[, default])
    
    Return the next item from the iterator. If default is given and the iterator
    is exhausted, it is returned instead of raising StopIteration.
    """

    # args = []
    # if default is not _SENTINEL:
    #     args.append(default)
    try:
        try:
            return iterator.__next__()
        except AttributeError:
            try:
                return iterator.next()
            except AttributeError:
                raise TypeError("'{0}' object is not an iterator".format(
                                           iterator.__class__.__name__))
    except StopIteration as e:
        if default is _SENTINEL:
            raise e
        else:
            return default


__all__ = ['newnext']




from abc import ABCMeta

from _AposA8Align import AposA8Align
from _AposA8CommandType import AposA8CommandType
from _AposA8PrintLine import AposA8PrintLine
from _AposA8Helpers import APOS_HASH, append_items_header, append_and_count_sale_items, APOS_DASH, add_loyalty_custom_text, \
    append_whopper_wifi_code, append_garena_coupons
from _AposA8Constants import AposA8Constants
from printer import FiscalPrinter


AMOUNT_TEMPLATE = "%-36s   R$%7.2f"


class AposA8FiscalPrinter(FiscalPrinter):
    __metaclass__ = ABCMeta

    def __init__(self, mbcontext, print_customer_name, cliche_helper, customer_text_field):

        super(AposA8FiscalPrinter, self).__init__(mbcontext, print_customer_name, cliche_helper)
        self.customer_text_field = customer_text_field

    def print_cancel(self, posid, data):
        # not implemented
        pass

    def _append_total(self, report, order, tenders, items_amount):
        qty_total = "%-36s         %3d" % ("QTD. TOTAL DE ITENS", items_amount)
        report.append(AposA8PrintLine(AposA8Align.CENTER, qty_total, AposA8CommandType.PRINT_TEXT))

        total_amount_value = float(order.get("totalAmount")) + float(order.get("taxTotal"))

        total_amount_text = AMOUNT_TEMPLATE % ("VALOR TOTAL R$", total_amount_value)
        report.append(AposA8PrintLine(AposA8Align.CENTER, total_amount_text, AposA8CommandType.PRINT_TEXT))

        discount_amount_value = float(order.get("discountAmount"))
        discount = AMOUNT_TEMPLATE % ("DESCONTO", discount_amount_value)
        report.append(AposA8PrintLine(AposA8Align.CENTER, discount, AposA8CommandType.PRINT_TEXT))

        total_payable = AMOUNT_TEMPLATE % ("VALOR A PAGAR", total_amount_value - discount_amount_value)
        report.append(AposA8PrintLine(AposA8Align.CENTER, total_payable, AposA8CommandType.PRINT_TEXT))

        payment = "%-36s  %s" % ("FORMA DE PAGAMENTO", "Valor Pago")
        report.append(AposA8PrintLine(AposA8Align.CENTER, payment, AposA8CommandType.PRINT_TEXT))
        for tender in tenders:
            tender_descr = self.get_payment_description(self.mb_context, tender)
            tender_amount = tender.amount + tender.change
            tender_text = AMOUNT_TEMPLATE % (tender_descr, tender_amount)
            report.append(AposA8PrintLine(AposA8Align.CENTER, tender_text, AposA8CommandType.PRINT_TEXT))

        # Get "Change" (sixth value of tender tuple) from last tender in tenders
        change = tenders[len(tenders) - 1].change
        if change > 0:
            order_voided = order.get("state") == "VOIDED"
            factor = -1 if order_voided else 1

            change_text = AMOUNT_TEMPLATE % ("Troco", change * factor)
            report.append(AposA8PrintLine(AposA8Align.CENTER, change_text, AposA8CommandType.PRINT_TEXT))

    def _append_customer_name(self, report, order, customer_name):
        if not self.print_customer_name:
            return

        char_amount = 21
        content = (customer_name or '')[:(char_amount - 2)] or order.get('orderId')[-3:]
        panel_call_text = "PAINEL DE RETIRADA - VOCE SERA CHAMADO POR:"
        report.append(AposA8PrintLine(AposA8Align.CENTER, panel_call_text, AposA8CommandType.PRINT_TEXT))

        report.append(APOS_HASH)
        report.append(AposA8PrintLine(AposA8Align.CENTER, content.upper(), AposA8CommandType.PRINT_TEXT))
        report.append(APOS_HASH)

    def _append_customer_text_field(self, report):
        if self.customer_text_field != "":
            for customer_text in self.customer_text_field.split('\\n'):
                report.append(
                    AposA8PrintLine(AposA8Align.CENTER, customer_text.replace("\\n", ""), AposA8CommandType.PRINT_TEXT))
            report.append(
                AposA8PrintLine(AposA8Align.CENTER, AposA8Constants.SEPARATOR_DASH, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_satisfaction_survey(report, model, satisfaction_survey_code):
        if satisfaction_survey_code is not None and model is not None:
            report.append(APOS_DASH)
            ssc = satisfaction_survey_code
            satisfaction_survey_code = '-'.join(ssc[i:i + 3] for i in range(0, len(ssc), 3))
            report.append(AposA8PrintLine(AposA8Align.CENTER, satisfaction_survey_code, AposA8CommandType.PRINT_TEXT))
            report.append(APOS_DASH)

    def _append_coupon_body(self, customer_name, model, order, report, tenders, whopper_wifi_code, tip_config):
        append_items_header(report)
        items_amount = append_and_count_sale_items(report, order, model, tip_config)
        report.append(APOS_DASH)
        self._append_total(report, order, tenders, items_amount)
        report.append(APOS_DASH)
        self._append_customer_name(report, order, customer_name)
        append_whopper_wifi_code(report, model, whopper_wifi_code)
        self._append_satisfaction_survey(report, model, self.get_satisfaction_survey_code(order))
        append_garena_coupons(self.mb_context, order, report)

    def _append_cliche(self, report):
        for line in self.cliche_helper.get_cliche():
            report.append(AposA8PrintLine(AposA8Align.CENTER, line, AposA8CommandType.PRINT_TEXT))
        report.append(APOS_DASH)






from production.box import OrderSequencerBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class OrderSequencerBoxBuilder(BoxBuilder):
    def __init__(self, product_repository, loggers):
        super(OrderSequencerBoxBuilder, self).__init__(loggers)
        self.product_repository = product_repository
        self.pod_types = []
        self.invert = False

    def build(self, config):
        # type: (BoxConfiguration) -> OrderSequencerBox
        if u"Sorters" not in config.extra_config:
            raise Exception("Sequencer without Sorters: {}".format(config.name))

        sorted_sorters = self.get_sorted_sorters(config.extra_config[u"Sorters"])

        if u"PodTypes" in config.extra_config:
            self.pod_types = config.extra_config[u"PodTypes"]
        if u"Invert" in config.extra_config:
            self.invert = (config.extra_config[u"Invert"].lower() or "") == "true"

        sorters = []
        for sorter_config in sorted_sorters:
            sorters.append(self.build_sorter(sorter_config))

        return OrderSequencerBox(
            config.name,
            config.sons,
            sorters,
            self.get_logger(config)
        )

    def get_sorted_sorters(self, sorters):
        sorters_list = []
        for sorter in sorters.keys():
            sorters_list.append((sorter, sorters[sorter]))

        return sorted(sorters_list, cmp=self.sorter_comparator)

    @staticmethod
    def sorter_comparator(s1, s2):
        return int(s1[1]) - int(s2[1])

    def build_sorter(self, sorter):
        if sorter[0] == "DoneInFrontSorter":
            return OrderSequencerBox.DoneInFrontSorter()
        elif sorter[0] == "LastModifiedSorter":
            return OrderSequencerBox.LastModifiedSorter(self.invert)
        elif sorter[0] == "DisplayTimeSorter":
            return OrderSequencerBox.DisplayTimeSorter(self.invert)
        elif sorter[0] == "PriorityCustomerSorter":
            return OrderSequencerBox.PriorityCustomerSorter()
        elif sorter[0] == "OrderIdSorter":
            return OrderSequencerBox.OrderIdSorter()
        elif sorter[0] == "DoneSorter":
            return OrderSequencerBox.DoneSorter()
        elif sorter[0] == "PodTypeSorter":
            return OrderSequencerBox.PodTypeSorter(
                self.pod_types[sorter[0]].split("|") if sorter[0] in self.pod_types else []
            )
        else:
            raise Exception("Not implemented sorter: {}".format(sorter))



from json import JSONEncoder

from ._StaticConfig import StaticConfig


class StaticConfigJsonEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, StaticConfig):
            return {
                "timeToAlertTableOpened": o.time_to_alert_table_opened,
                "timeToAlertTableIsIdle": o.time_to_alert_table_is_idle,
                "timeToAlertTableIsIdleWarning": o.time_to_alert_table_is_idle_warning,
                "timeToAlertRecallDeliveryIsIdle": o.time_to_alert_recall_delivery_is_idle,
                "recallButton": o.recall_button,
                "operatorButton": o.operator_button,
                "specialMenus": o.special_menus,
                "enabledTags": o.enabled_tags,
                "screenTimeout": o.screen_timeout,
                "cancelTimeoutWindow": o.cancel_timeout_window,
                "showInDashboard": o.show_in_dashboard,
                "productsScreenDimensions": o.products_screen_dimensions,
                "spinnerConfig": o.spinner_config,
                "enableTabBtns": o.enable_tab_btns,
                "enablePreStartSale": o.enable_pre_start_sale,
                "availableSaleTypes": o.available_sale_types,
                "fetchStoredOrdersTimeout": o.fetch_stored_orders_timeout,
                "tenderTypes": o.tender_types,
                "priceOverrideEnabled": o.price_override_enabled,
                "canOpenTableFromAnotherOperator": o.can_open_table_from_another_operator,
                "canEditOrder": o.can_edit_order,
                "satInfo": o.sat_info,
                "navigationOptions": o.navigation_config,
                "showCashInAndCashOut": o.show_cash_in_and_cash_out,
                "showVoidClosedOrder": o.show_void_closed_order,
                "minLevelNeededToSeeAllTables": o.min_level_needed_to_see_all_tables,
                "totemConfigurations": o.totem_config,
                "cashPaymentEnabled": o.cash_payment_enabled,
                "couponsEnabled": o.coupons_enabled,
                "discountsEnabled": o.discounts_enabled,
                "billPaymentEnabled": o.bill_payment_enabled,
                "specialModifiers": o.special_modifiers,
                "showRupturedProducts": o.show_ruptured_products,
                "posNavigation": o.pos_navigation,
                "remoteOrderStatus": o.remote_order_status,
                "deliverySound": o.delivery_sound,
                'deliveryAddress': o.delivery_address,
                "chatEnabled": o.chat_enabled,
                "servicesPeriod": o.services_period,
                "menuTest": o.menu_test,
                "preAccountWithDiscount": o.pre_account_with_discount,
                "mainNavigation": o.main_navigation
            }

        return super(StaticConfigJsonEncoder, self).default(o)



# -*- coding: utf-8 -*-
import json
import logging

from actions.delivery.delivery_address_information import insert_or_update_customer
from sysactions import action, show_any_dialog, show_messagebox
from custom_sysactions import block_action_if_order_items_not_resolved

logger = logging.getLogger("PosActions")


@action
@block_action_if_order_items_not_resolved
def delivery_address(pos_id):
    try:
        customer = show_any_dialog(pos_id, "deliveryAddress", "", "", "", "", "", 6000000, "", "", "", "", "", False)
        customer_json = json.loads(customer)
        if customer_json:
            insert_or_update_customer(pos_id, customer_json)
            return True
    except Exception as _:
        logger.exception("Error inserting or updating customer")
        show_messagebox(pos_id, message="$CUSTOMER_ERROR_ON_UPDATE")

    return False



# Wrapper module for _socket, providing some additional facilities
# implemented in Python.

"""\
This module provides socket operations and some related functions.
On Unix, it supports IP (Internet Protocol) and Unix domain sockets.
On other systems, it only supports IP. Functions specific for a
socket are available as methods of the socket object.

Functions:

socket() -- create a new socket object
socketpair() -- create a pair of new socket objects [*]
fromfd() -- create a socket object from an open file descriptor [*]
fromshare() -- create a socket object from data received from socket.share() [*]
gethostname() -- return the current hostname
gethostbyname() -- map a hostname to its IP number
gethostbyaddr() -- map an IP number or hostname to DNS info
getservbyname() -- map a service name and a protocol name to a port number
getprotobyname() -- map a protocol name (e.g. 'tcp') to a number
ntohs(), ntohl() -- convert 16, 32 bit int from network to host byte order
htons(), htonl() -- convert 16, 32 bit int from host to network byte order
inet_aton() -- convert IP addr string (123.45.67.89) to 32-bit packed format
inet_ntoa() -- convert 32-bit packed format IP to string (123.45.67.89)
socket.getdefaulttimeout() -- get the default timeout value
socket.setdefaulttimeout() -- set the default timeout value
create_connection() -- connects to an address, with an optional timeout and
                       optional source address.

 [*] not available on all platforms!

Special objects:

SocketType -- type object for socket objects
error -- exception raised for I/O errors
has_ipv6 -- boolean value indicating if IPv6 is supported

Integer constants:

AF_INET, AF_UNIX -- socket domains (first argument to socket() call)
SOCK_STREAM, SOCK_DGRAM, SOCK_RAW -- socket types (second argument)

Many other constants may be defined; these may be used in calls to
the setsockopt() and getsockopt() methods.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future.builtins import super

import _socket
from _socket import *

import os, sys, io

try:
    import errno
except ImportError:
    errno = None
EBADF = getattr(errno, 'EBADF', 9)
EAGAIN = getattr(errno, 'EAGAIN', 11)
EWOULDBLOCK = getattr(errno, 'EWOULDBLOCK', 11)

__all__ = ["getfqdn", "create_connection"]
__all__.extend(os._get_exports_list(_socket))


_realsocket = socket

# WSA error codes
if sys.platform.lower().startswith("win"):
    errorTab = {}
    errorTab[10004] = "The operation was interrupted."
    errorTab[10009] = "A bad file handle was passed."
    errorTab[10013] = "Permission denied."
    errorTab[10014] = "A fault occurred on the network??" # WSAEFAULT
    errorTab[10022] = "An invalid operation was attempted."
    errorTab[10035] = "The socket operation would block"
    errorTab[10036] = "A blocking operation is already in progress."
    errorTab[10048] = "The network address is in use."
    errorTab[10054] = "The connection has been reset."
    errorTab[10058] = "The network has been shut down."
    errorTab[10060] = "The operation timed out."
    errorTab[10061] = "Connection refused."
    errorTab[10063] = "The name is too long."
    errorTab[10064] = "The host is down."
    errorTab[10065] = "The host is unreachable."
    __all__.append("errorTab")


class socket(_socket.socket):

    """A subclass of _socket.socket adding the makefile() method."""

    __slots__ = ["__weakref__", "_io_refs", "_closed"]

    def __init__(self, family=AF_INET, type=SOCK_STREAM, proto=0, fileno=None):
        if fileno is None:
            _socket.socket.__init__(self, family, type, proto)
        else:
            _socket.socket.__init__(self, family, type, proto, fileno)
        self._io_refs = 0
        self._closed = False

    def __enter__(self):
        return self

    def __exit__(self, *args):
        if not self._closed:
            self.close()

    def __repr__(self):
        """Wrap __repr__() to reveal the real class name."""
        s = _socket.socket.__repr__(self)
        if s.startswith("<socket object"):
            s = "<%s.%s%s%s" % (self.__class__.__module__,
                                self.__class__.__name__,
                                getattr(self, '_closed', False) and " [closed] " or "",
                                s[7:])
        return s

    def __getstate__(self):
        raise TypeError("Cannot serialize socket object")

    def dup(self):
        """dup() -> socket object

        Return a new socket object connected to the same system resource.
        """
        fd = dup(self.fileno())
        sock = self.__class__(self.family, self.type, self.proto, fileno=fd)
        sock.settimeout(self.gettimeout())
        return sock

    def accept(self):
        """accept() -> (socket object, address info)

        Wait for an incoming connection.  Return a new socket
        representing the connection, and the address of the client.
        For IP sockets, the address info is a pair (hostaddr, port).
        """
        fd, addr = self._accept()
        sock = socket(self.family, self.type, self.proto, fileno=fd)
        # Issue #7995: if no default timeout is set and the listening
        # socket had a (non-zero) timeout, force the new socket in blocking
        # mode to override platform-specific socket flags inheritance.
        if getdefaulttimeout() is None and self.gettimeout():
            sock.setblocking(True)
        return sock, addr

    def makefile(self, mode="r", buffering=None, **_3to2kwargs):
        """makefile(...) -> an I/O stream connected to the socket

        The arguments are as for io.open() after the filename,
        except the only mode characters supported are 'r', 'w' and 'b'.
        The semantics are similar too.  (XXX refactor to share code?)
        """
        if 'newline' in _3to2kwargs: newline = _3to2kwargs['newline']; del _3to2kwargs['newline']
        else: newline = None
        if 'errors' in _3to2kwargs: errors = _3to2kwargs['errors']; del _3to2kwargs['errors']
        else: errors = None
        if 'encoding' in _3to2kwargs: encoding = _3to2kwargs['encoding']; del _3to2kwargs['encoding']
        else: encoding = None
        for c in mode:
            if c not in ("r", "w", "b"):
                raise ValueError("invalid mode %r (only r, w, b allowed)")
        writing = "w" in mode
        reading = "r" in mode or not writing
        assert reading or writing
        binary = "b" in mode
        rawmode = ""
        if reading:
            rawmode += "r"
        if writing:
            rawmode += "w"
        raw = SocketIO(self, rawmode)
        self._io_refs += 1
        if buffering is None:
            buffering = -1
        if buffering < 0:
            buffering = io.DEFAULT_BUFFER_SIZE
        if buffering == 0:
            if not binary:
                raise ValueError("unbuffered streams must be binary")
            return raw
        if reading and writing:
            buffer = io.BufferedRWPair(raw, raw, buffering)
        elif reading:
            buffer = io.BufferedReader(raw, buffering)
        else:
            assert writing
            buffer = io.BufferedWriter(raw, buffering)
        if binary:
            return buffer
        text = io.TextIOWrapper(buffer, encoding, errors, newline)
        text.mode = mode
        return text

    def _decref_socketios(self):
        if self._io_refs > 0:
            self._io_refs -= 1
        if self._closed:
            self.close()

    def _real_close(self, _ss=_socket.socket):
        # This function should not reference any globals. See issue #808164.
        _ss.close(self)

    def close(self):
        # This function should not reference any globals. See issue #808164.
        self._closed = True
        if self._io_refs <= 0:
            self._real_close()

    def detach(self):
        """detach() -> file descriptor

        Close the socket object without closing the underlying file descriptor.
        The object cannot be used after this call, but the file descriptor
        can be reused for other purposes.  The file descriptor is returned.
        """
        self._closed = True
        return super().detach()

def fromfd(fd, family, type, proto=0):
    """ fromfd(fd, family, type[, proto]) -> socket object

    Create a socket object from a duplicate of the given file
    descriptor.  The remaining arguments are the same as for socket().
    """
    nfd = dup(fd)
    return socket(family, type, proto, nfd)

if hasattr(_socket.socket, "share"):
    def fromshare(info):
        """ fromshare(info) -> socket object

        Create a socket object from a the bytes object returned by
        socket.share(pid).
        """
        return socket(0, 0, 0, info)

if hasattr(_socket, "socketpair"):

    def socketpair(family=None, type=SOCK_STREAM, proto=0):
        """socketpair([family[, type[, proto]]]) -> (socket object, socket object)

        Create a pair of socket objects from the sockets returned by the platform
        socketpair() function.
        The arguments are the same as for socket() except the default family is
        AF_UNIX if defined on the platform; otherwise, the default is AF_INET.
        """
        if family is None:
            try:
                family = AF_UNIX
            except NameError:
                family = AF_INET
        a, b = _socket.socketpair(family, type, proto)
        a = socket(family, type, proto, a.detach())
        b = socket(family, type, proto, b.detach())
        return a, b


_blocking_errnos = set([EAGAIN, EWOULDBLOCK])

class SocketIO(io.RawIOBase):

    """Raw I/O implementation for stream sockets.

    This class supports the makefile() method on sockets.  It provides
    the raw I/O interface on top of a socket object.
    """

    # One might wonder why not let FileIO do the job instead.  There are two
    # main reasons why FileIO is not adapted:
    # - it wouldn't work under Windows (where you can't used read() and
    #   write() on a socket handle)
    # - it wouldn't work with socket timeouts (FileIO would ignore the
    #   timeout and consider the socket non-blocking)

    # XXX More docs

    def __init__(self, sock, mode):
        if mode not in ("r", "w", "rw", "rb", "wb", "rwb"):
            raise ValueError("invalid mode: %r" % mode)
        io.RawIOBase.__init__(self)
        self._sock = sock
        if "b" not in mode:
            mode += "b"
        self._mode = mode
        self._reading = "r" in mode
        self._writing = "w" in mode
        self._timeout_occurred = False

    def readinto(self, b):
        """Read up to len(b) bytes into the writable buffer *b* and return
        the number of bytes read.  If the socket is non-blocking and no bytes
        are available, None is returned.

        If *b* is non-empty, a 0 return value indicates that the connection
        was shutdown at the other end.
        """
        self._checkClosed()
        self._checkReadable()
        if self._timeout_occurred:
            raise IOError("cannot read from timed out object")
        while True:
            try:
                return self._sock.recv_into(b)
            except timeout:
                self._timeout_occurred = True
                raise
            # except InterruptedError:
            #     continue
            except error as e:
                if e.args[0] in _blocking_errnos:
                    return None
                raise

    def write(self, b):
        """Write the given bytes or bytearray object *b* to the socket
        and return the number of bytes written.  This can be less than
        len(b) if not all data could be written.  If the socket is
        non-blocking and no bytes could be written None is returned.
        """
        self._checkClosed()
        self._checkWritable()
        try:
            return self._sock.send(b)
        except error as e:
            # XXX what about EINTR?
            if e.args[0] in _blocking_errnos:
                return None
            raise

    def readable(self):
        """True if the SocketIO is open for reading.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return self._reading

    def writable(self):
        """True if the SocketIO is open for writing.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return self._writing

    def seekable(self):
        """True if the SocketIO is open for seeking.
        """
        if self.closed:
            raise ValueError("I/O operation on closed socket.")
        return super().seekable()

    def fileno(self):
        """Return the file descriptor of the underlying socket.
        """
        self._checkClosed()
        return self._sock.fileno()

    @property
    def name(self):
        if not self.closed:
            return self.fileno()
        else:
            return -1

    @property
    def mode(self):
        return self._mode

    def close(self):
        """Close the SocketIO object.  This doesn't close the underlying
        socket, except if all references to it have disappeared.
        """
        if self.closed:
            return
        io.RawIOBase.close(self)
        self._sock._decref_socketios()
        self._sock = None


def getfqdn(name=''):
    """Get fully qualified domain name from name.

    An empty argument is interpreted as meaning the local host.

    First the hostname returned by gethostbyaddr() is checked, then
    possibly existing aliases. In case no FQDN is available, hostname
    from gethostname() is returned.
    """
    name = name.strip()
    if not name or name == '0.0.0.0':
        name = gethostname()
    try:
        hostname, aliases, ipaddrs = gethostbyaddr(name)
    except error:
        pass
    else:
        aliases.insert(0, hostname)
        for name in aliases:
            if '.' in name:
                break
        else:
            name = hostname
    return name


# Re-use the same sentinel as in the Python stdlib socket module:
from socket import _GLOBAL_DEFAULT_TIMEOUT
# Was: _GLOBAL_DEFAULT_TIMEOUT = object()


def create_connection(address, timeout=_GLOBAL_DEFAULT_TIMEOUT,
                      source_address=None):
    """Connect to *address* and return the socket object.

    Convenience function.  Connect to *address* (a 2-tuple ``(host,
    port)``) and return the socket object.  Passing the optional
    *timeout* parameter will set the timeout on the socket instance
    before attempting to connect.  If no *timeout* is supplied, the
    global default timeout setting returned by :func:`getdefaulttimeout`
    is used.  If *source_address* is set it must be a tuple of (host, port)
    for the socket to bind as a source address before making the connection.
    An host of '' or port 0 tells the OS to use the default.
    """

    host, port = address
    err = None
    for res in getaddrinfo(host, port, 0, SOCK_STREAM):
        af, socktype, proto, canonname, sa = res
        sock = None
        try:
            sock = socket(af, socktype, proto)
            if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
                sock.settimeout(timeout)
            if source_address:
                sock.bind(source_address)
            sock.connect(sa)
            return sock

        except error as _:
            err = _
            if sock is not None:
                sock.close()

    if err is not None:
        raise err
    else:
        raise error("getaddrinfo returns an empty list")



import base64
import cPickle as Pickle
from Queue import Queue, Empty
from threading import Thread

import production.repository
from mw_helper import BaseRepository
from persistence import Connection
from production.model import ProductionOrder
from typing import List


class ProductionRepository(BaseRepository, production.repository.ProductionRepository):
    def __init__(self, mb_context, logger):
        super(ProductionRepository, self).__init__(mb_context)
        self.logger = logger

        self.order_queue = Queue()
        self.running = True

        self.processor_thread = Thread(target=self.consume_queue)
        self.processor_thread.daemon = True
        self.processor_thread.start()

    def consume_queue(self):
        while self.running:
            order_id = None
            try:
                order_to_save = self.order_queue.get(True, 5)
                order_id = order_to_save.order_id
                self.save_order_async(order_to_save)
            except Empty:
                continue
            except BaseException as _:  # noqa
                self.logger.exception("Error saving order: {}".format(order_id))

    def stop(self):
        self.running = False

    def save_order(self, order):
        self.order_queue.put(order)

    def save_order_async(self, order):
        def func(conn):
            # type: (Connection) -> None
            order_data = base64.b64encode(Pickle.dumps(order, Pickle.HIGHEST_PROTOCOL))
            conn.query(insert_query.format(order.order_id, order_data, order.created_at, 0))

        self.execute_with_connection(func)

    def get_all_orders(self):
        def func(conn):
            # type: (Connection) -> List[ProductionOrder]
            orders = []
            for row in conn.select(select_query):
                order_data = base64.b64decode(row.get_entry(0))
                order = Pickle.loads(order_data)
                orders.append(order)

            return orders

        return self.execute_with_connection(func)

    def get_orders_to_purge(self, limit_date):
        def func(conn):
            # type: (Connection) -> List[int]
            orders = []
            formatted_limit_date = limit_date.strftime("%Y-%m-%dT%H:%M:%S")
            for row in conn.select(get_orders_to_purge_query.format(formatted_limit_date)):
                order_id = int(row.get_entry("OrderId"))
                orders.append(order_id)

            return orders

        return self.execute_with_connection(func)

    def purge_orders(self, orders):
        def func(conn):
            # type: (Connection) -> None
            orders_list = [str(order) for order in orders]
            query = purge_orders_query.format(",".join(orders_list))
            conn.query(query)

        return self.execute_with_connection(func)

insert_query = """
INSERT OR REPLACE INTO ProductionOrders
(OrderId, Object, CreatedAt, Purged)
VALUES
({}, '{}', '{}', {})
"""

select_query = """
SELECT Object FROM ProductionOrders WHERE PURGED = 0
"""

get_orders_to_purge_query = """
SELECT OrderId FROM ProductionOrders WHERE CreatedAt < '{}'
"""

purge_orders_query = """
DELETE FROM ProductionOrders WHERE OrderId IN ({})
"""



# -*- coding: utf-8 -*-

from datetime import datetime

from msgbus import MBEasyContext
from old_helper import BaseRepository
from persistence import Connection
from pyscriptscache import cache as _cache
from systools import sys_log_error
from typing import List, Dict, Optional

from ..model import Order, OrderItem

PRODUCT_OPTIONS_CACHE = []


class OrderRepository(BaseRepository):

    def __init__(self, mbcontext, pos_list):
        # type: (MBEasyContext, List[int]) -> None
        super(OrderRepository, self).__init__(mbcontext)
        self.pos_list = pos_list

    def get_paid_orders_by_real_date(self, initial_date, end_date, operatorid=""):
        # type: (datetime, datetime, Optional[str]) -> List[Order]
        return self._get_paid_orders(initial_date, end_date, True, operatorid)

    def get_paid_orders_by_business_period(self, initial_date, end_date, operatorid=""):
        # type: (datetime, datetime, Optional[str]) -> List[Order]
        return self._get_paid_orders(initial_date, end_date, False, operatorid)

    def get_orders_items(self, paid_orders):
        # type: (List[Order]) -> List[OrderItem]
        def inner_func(conn):
            # type: (Connection) -> List[OrderItem]
            query = self._OrderItemsQuery

            order_list = map(lambda order: order.order_id, sorted(paid_orders))
            order_list = ','.join(map(str, order_list))
            query = query.format(order_list)

            all_order_items = {}  # type: Dict[int, OrderItem]
            try:
                order_items = [(int(x.get_entry(0)), float(x.get_entry(1)), str(x.get_entry(2))) for x in conn.select(query)]

                for order_tuple in order_items:
                    pcode = order_tuple[0]
                    quantity = order_tuple[1]
                    item_id = order_tuple[2]

                    if quantity == 0:
                        continue

                    cache_unit_price, cache_add_price, cache_sub_price = 0, 0, 0

                    if _cache.get_best_price(item_id, str(pcode)):
                        cache_unit_price, cache_add_price, cache_sub_price = _cache.get_best_price(item_id, str(pcode))

                    price = float(cache_unit_price or 0) or float(cache_add_price or 0)
                    if pcode not in all_order_items.keys():
                        order_item = OrderItem(u"", quantity, pcode, price)
                        all_order_items[pcode] = order_item
                    else:
                        all_order_items[pcode].quantity += quantity

                return list(all_order_items.values())
            except Exception as e:
                sys_log_error("get order error: %s" % str(e))
                return list({"error": "error"}.values())

        return self.execute_parallel_in_all_databases_returning_flat_list(inner_func, self.pos_list)

    def _get_paid_orders(self, initial_date, end_date, real_date, operator_id):
        # type: (Optional[datetime], Optional[datetime], Optional[bool], Optional[int]) -> List[Order]
        def inner_func(conn):
            # type: (Connection) -> List[Order]

            try:
                if real_date:
                    query = self._PaidOrderByRealDate
                    date_format = "%Y-%m-%d"
                else:
                    query = self._PaidOrderByBusinessPeriod
                    date_format = "%Y%m%d"

                query = query.format(initial_date.strftime(date_format), end_date.strftime(date_format), int(operator_id))
                if operator_id in ["", "0", "None"]:
                    query = query.replace("user={},".format(int(operator_id)), "user=")

                all_orders = {}  # type: Dict[int, Order]
                order_with_payments = [(int(x.get_entry(0)),
                                        int(x.get_entry(1)),
                                        datetime.strptime(x.get_entry(2), "%Y%m%d"),
                                        float(x.get_entry(3)),
                                        int(x.get_entry(4)),
                                        float(x.get_entry(5))) for x in conn.select(query)]

                for order_tuple in order_with_payments:
                    order_id = order_tuple[0]
                    order_state = order_tuple[1]
                    order_date = order_tuple[2]

                    if order_id not in all_orders:
                        order = Order(order_id, order_date, order_state)
                        all_orders[order_id] = order

                return list(all_orders.values())
            except:
                return list()

        return self.execute_parallel_in_all_databases_returning_flat_list(inner_func, self.pos_list)

    _PaidOrderByBusinessPeriod = \
"""select o.OrderId, o.StateId, o.BusinessPeriod, o.TotalGross, ot.TenderId, ot.TenderAmount - coalesce(ot.ChangeAmount, 0)
from Orders o
inner join OrderTender ot
on o.OrderId = ot.OrderId
where o.BusinessPeriod >= '{0}' and o.BusinessPeriod <= '{1}' and o.StateId = 5 and o.SessionId like '%user={2},%'"""

    _PaidOrderByRealDate = \
"""select o.OrderId, o.StateId, o.BusinessPeriod, o.TotalGross, ot.TenderId, ot.TenderAmount - coalesce(ot.ChangeAmount, 0)
from Orders o
inner join OrderTender ot
on o.OrderId = ot.OrderId
inner join 
(
	-- Busca as Orders do dia que estamos interessadosh
	select OrderId, strftime('%Y-%m-%d', Value, 'localtime') OrderDate
	from OrderCustomProperties
	where key = 'FISCALIZATION_DATE' and strftime('%Y-%m-%d', Value, 'localtime') >= '{0}' and strftime('%Y-%m-%d', Value, 'localtime') <= '{1}'
) od
on o.OrderId = od.OrderId
where o.SessionId like '%user={2},%'"""


    _OrderItemsQuery = """\
    SELECT * FROM (
    SELECT PartCode, SUM(OrderedQty) as SOMA, ItemId, PriceListId1
        FROM
            (SELECT 
			t.PartCode,
			CASE WHEN t.DefaultQty IS NULL
				THEN t.OrderedQty 
				ELSE
					CASE WHEN t.OrderedQty IS NULL
						THEN t.DefaultQty 
					ELSE
						CASE WHEN t.DefaultQty > t.OrderedQty
						THEN 0
						ELSE t.OrderedQty - t.DefaultQty
						END
					END
				END AS OrderedQty,
			t.ItemId,
			t.OrderId,
			s.PriceListId1
			FROM OrderItem t
			JOIN OrderItem u ON t.OrderId = u.OrderId AND t.LineNumber = u.LineNumber
			JOIN Orders s ON t.OrderId = s.OrderId
			JOIN ProductKernelParams pkp ON pkp.ProductCode = t.PartCode
			WHERE s.StateId = 5
			    AND u.Level = '0' 
			    AND t.OrderId IN ({0})
			    AND pkp.ProductType <> 1
			    AND t.PartCode NOT IN (
			        SELECT ProductCode FROM ProductTags
                    WHERE LOWER(Tag) = 'cfh=true'
                    UNION
                    SELECT ProductCode FROM ProductCustomParams
                    WHERE LOWER(CustomParamId) = 'cfh' AND LOWER(CustomParamValue) = 'true'
			    )
			    AND COALESCE(t.OrderedQty, t.DefaultQty) > 0) T
    GROUP BY PartCode, ItemId) where SOMA is not null
    """



from ._TipReportHeaderDto import TipReportHeaderDto
from ._TipReportBodyDto import TipReportBodyDto


class TipReportDto(object):
    def __init__(self, header, body):
        # type: (TipReportHeaderDto, TipReportBodyDto) -> None
        self.header = header
        self.body = body



from xml.etree import cElementTree as Etree

from model.exception import InvalidOrderEventException
from model.log import LogEntry
from _EventProcessor import EventProcessor


def _get_log_entry_from_order_event(data, event_type):
    try:
        order = _check_mandatory_information(data)
    except InvalidOrderEventException:
        return None

    pos_id = order.get("posId", order.get("posid"))
    if pos_id is None:
        pos_id = order.get("instance") or -1

    return LogEntry(
        pos_id=pos_id,
        order_id=order.get("orderId", order.get("orderid")),
        business_period=order.get("businessPeriod", order.get("period")),
        event_type=event_type,
        created_at_gmt=order.get("createdAtGMT"),
        session_id=order.get("sessionId", order.get("sessionid")),
        event_data=Etree.tostring(order)
    )


def _check_mandatory_information(data):
    event = Etree.fromstring(data)
    order = event.find(".//Order")
    if order is None or order.get("orderId") is None:
        raise InvalidOrderEventException(data)

    return order


class OrderEventProcessor(EventProcessor):
    def parse_data(self, data, event_type):
        # type: (str, str) -> LogEntry
        return _get_log_entry_from_order_event(data, event_type)



from ._BodyNotJson import BodyNotJson
from ._InteractorWithMoreThanOneParameterOnExecute import InteractorWithMoreThanOneParameterOnExecute
from ._InteractorWithoutExecuteMethod import InteractorWithoutExecuteMethod
from ._ParameterSentToNoParameterInteractor import ParameterSentToNoParameterInteractor
from ._PropertyNotSent import PropertyNotSent
from ._PropertyError import PropertyError



from logging import Logger

from production.command import CommandType
from production.box import ViewBox
from production.model.request import ToggleTagLineRequest, SaleLine
from typing import List, Dict

from ._CommandProcessor import CommandProcessor


class ToggleTagLineProcessor(CommandProcessor):
    def __init__(self, view_boxes, loggers):
        # type: (List[ViewBox], Dict[str, Logger]) -> None

        super(ToggleTagLineProcessor, self).__init__(view_boxes)

        self.error_logger = loggers["ERROR"]

    def parse_data(self, data):
        try:
            parts = data.split('\0')
            lines = parts[2].split(";")
            sale_lines = []
            for line in lines:
                line_parts = line.split('-')
                sale_lines.append(SaleLine(line_parts[0], line_parts[1], line_parts[2], line_parts[3]))

            return ToggleTagLineRequest(parts[0], parts[1], sale_lines, parts[3])
        except Exception as _:
            error_message = "Error parsing data in ToggleTagLineProcessor. Data: {}"
            self.error_logger.error(error_message.format("/".join(data.split('\0'))))
            raise

    def get_command_type(self):
        return CommandType.toggle_tag_line



# -*- coding: utf-8 -*-

from sysactions import show_confirmation, StopAction, action, show_messagebox
from bustoken import TK_REMOTE_ORDER_CANCEL_LOGISTIC
from msgbus import TK_SYS_ACK

from .. import mb_context
from model.customexception import LogisticCancelError


@action
def cancel_logistic_remote_order(pos_id, order_id=""):
    confirmation = show_confirmation(pos_id, "$LOGISTIC_CANCEL_CONFIRMATION")
    if not confirmation:
        raise StopAction()

    data = pos_id + "\0" + order_id

    msg = mb_context.MB_EasySendMessage("RemoteOrder", TK_REMOTE_ORDER_CANCEL_LOGISTIC, data=data, timeout=10000000)
    if msg.token == TK_SYS_ACK:
        show_messagebox(pos_id, "$LOGISTIC_CANCELED")
        return
    
    raise LogisticCancelError("$LOGISTIC_CANCELLED_ERROR")



"""Error classes used by simplejson
"""
__all__ = ['JSONDecodeError']


def linecol(doc, pos):
    lineno = doc.count('\n', 0, pos) + 1
    if lineno == 1:
        colno = pos + 1
    else:
        colno = pos - doc.rindex('\n', 0, pos)
    return lineno, colno


def errmsg(msg, doc, pos, end=None):
    lineno, colno = linecol(doc, pos)
    msg = msg.replace('%r', repr(doc[pos:pos + 1]))
    if end is None:
        fmt = '%s: line %d column %d (char %d)'
        return fmt % (msg, lineno, colno, pos)
    endlineno, endcolno = linecol(doc, end)
    fmt = '%s: line %d column %d - line %d column %d (char %d - %d)'
    return fmt % (msg, lineno, colno, endlineno, endcolno, pos, end)


class JSONDecodeError(ValueError):
    """Subclass of ValueError with the following additional properties:

    msg: The unformatted error message
    doc: The JSON document being parsed
    pos: The start index of doc where parsing failed
    end: The end index of doc where parsing failed (may be None)
    lineno: The line corresponding to pos
    colno: The column corresponding to pos
    endlineno: The line corresponding to end (may be None)
    endcolno: The column corresponding to end (may be None)

    """
    # Note that this exception is used from _speedups
    def __init__(self, msg, doc, pos, end=None):
        ValueError.__init__(self, errmsg(msg, doc, pos, end=end))
        self.msg = msg
        self.doc = doc
        self.pos = pos
        self.end = end
        self.lineno, self.colno = linecol(doc, pos)
        if end is not None:
            self.endlineno, self.endcolno = linecol(doc, end)
        else:
            self.endlineno, self.endcolno = None, None

    def __reduce__(self):
        return self.__class__, (self.msg, self.doc, self.pos, self.end)



from datetime import datetime

from production.box import CustomParamPrepTimeBox
from production.model import ProductionOrder
from production.repository import ProductRepository
from timeutil import RealClock


class GetOrderMaxPrepTimeInteractor(object):
    def __init__(self, product_repository):
        # type: (ProductRepository) -> None
        self.product_repository = product_repository

    def execute(self, production_order):
        # type: (ProductionOrder) -> int
        production_order.display_time = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
        custom_param_prep_time_box = CustomParamPrepTimeBox("", None, self.product_repository, RealClock())
        production_order = custom_param_prep_time_box.change_order(production_order)

        return production_order.max_prep_time
